using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using DOOM.WAD;



namespace DOOM.Levels {
    public partial class Level {

        public Point[] Vertices;
        public Wad.Line[] Lines;
        public Wad.Side[] Sides;
        public Wad.Sector[] Sectors;
        public TextureWallGroup[] Walls;
        public Wad.Segment[] Segments;
        public Wad.SubSector[] SubSectors;
        public Wad.Thing[] Things;
        public Wad.BspNode[] Nodes;
        public readonly string Name = "";

        public Wad WadFile;


        public Wad.Sector GetSectorAtPoint(Point Pt) {
            
            foreach (Wad.Sector S in this.Sectors) {


                bool IsTheOne = true;

                foreach (Wad.Segment G in S.ContainedSegments) {
                    Vector3 A = new Vector3(this.Vertices[G.StartVertex].X - Pt.X, this.Vertices[G.StartVertex].Y - Pt.Y, 0.0f);
                    Vector3 B = new Vector3(this.Vertices[G.EndVertex].X - Pt.X, this.Vertices[G.EndVertex].Y - Pt.Y, 0.0f);
                    Vector3 Result = Vector3.Cross(B, A);
                    if (Result.Z < 0) {
                        IsTheOne = false;
                        break;
                    }
                }
                if (IsTheOne) return S;
            }
            
            return null;
        }

        public Level(Wad WadFile, string LevelName) {
            this.WadFile = WadFile;

            int FindLevel = 0;
            for (; FindLevel < WadFile.LumpCount; ++FindLevel) {
                if (WadFile.Lumps[FindLevel].Type == Wad.LumpType.Marker && WadFile.Lumps[FindLevel].Name == LevelName) { //   ?
                    break;
                }                
            }
            if (FindLevel == WadFile.LumpCount) throw new Exception("Level not found.");

            this.Name = LevelName;

            Wad.Lump VertexLump = null;
            Wad.Lump LineDefLump = null;
            Wad.Lump SideDefLump = null;
            Wad.Lump SectorLump = null;
            Wad.Lump SubSectorLump = null;
            Wad.Lump SegmentLump = null;
            Wad.Lump ThingLump = null;
            Wad.Lump NodeLump = null;

            for (int i = FindLevel + 1; i < WadFile.LumpCount; ++i) {
                Wad.Lump DetectLump = WadFile.Lumps[i];
                if (DetectLump.Type == Wad.LumpType.Marker) break;
                switch (DetectLump.Type) {
                    case Wad.LumpType.VertexList:
                        if (VertexLump == null) VertexLump = DetectLump;
                        break;
                    case Wad.LumpType.LineDefinitions:
                        if (LineDefLump == null) LineDefLump = DetectLump;
                        break;
                    case Wad.LumpType.SideDefinitions:
                        if (SideDefLump == null) SideDefLump = DetectLump;
                        break;
                    case Wad.LumpType.SectorList:
                        if (SectorLump == null) SectorLump = DetectLump;
                        break;
                    case Wad.LumpType.SubsectorList:
                        if (SubSectorLump == null) SubSectorLump = DetectLump;
                        break;
                    case Wad.LumpType.SegmentList:
                        if (SegmentLump == null) SegmentLump = DetectLump;
                        break;
                    case Wad.LumpType.Things:
                        if (ThingLump == null) ThingLump = DetectLump;
                        break;
                    case Wad.LumpType.NodeList:
                        if (NodeLump == null) NodeLump = DetectLump;
                        break;
                }
            }

            if (VertexLump == null) throw new Exception("Could not find vertex lump.");
            if (LineDefLump == null) throw new Exception("Could not find linedef lump.");
            if (SideDefLump == null) throw new Exception("Could not find sidedef lump.");
            if (SectorLump == null) throw new Exception("Could not find sector lump.");
            if (SegmentLump == null) throw new Exception("Could not find segment lump.");
            if (SubSectorLump == null) throw new Exception("Could not find subsector lump.");
            if (ThingLump == null) throw new Exception("Could not find things lump.");
            if (NodeLump == null) throw new Exception("Could not find nodes lump.");


            this.Vertices = (Point[])VertexLump.DecodedData;
            this.Lines = (Wad.Line[])LineDefLump.DecodedData;
            this.Sides = (Wad.Side[])SideDefLump.DecodedData;
            this.Sectors = (Wad.Sector[])SectorLump.DecodedData;
            this.Segments = (Wad.Segment[])SegmentLump.DecodedData;
            this.SubSectors = (Wad.SubSector[])SubSectorLump.DecodedData;
            this.Things = (Wad.Thing[])ThingLump.DecodedData;
            this.Nodes = (Wad.BspNode[])NodeLump.DecodedData;

            // Index the line definitions

            foreach (Wad.Sector S in Sectors) {
                S.IndexLineDefs(this.Lines, this.Sides, this.Segments);           
            }

            // Now we need to encode the level into a series of vertex buffers,
            // one per texture.
            
            Hashtable AllWalls = new Hashtable();
            Hashtable WallTextures = new Hashtable();


            foreach (Wad.Side S in Sides) {

                if (S.MiddleTexture != null) {
                    object FindTexture = AllWalls[S.MiddleTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.MiddleTexture.ID] = new List<Wall>();
                        WallTextures[S.MiddleTexture.ID] = S.MiddleTexture.Texture;
                    }
                }
                if (S.UpperTexture != null) {
                    object FindTexture = AllWalls[S.UpperTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.UpperTexture.ID] = new List<Wall>();
                        WallTextures[S.UpperTexture.ID] = S.UpperTexture.Texture;
                    }
                }
                if (S.LowerTexture != null) {
                    object FindTexture = AllWalls[S.LowerTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.LowerTexture.ID] = new List<Wall>();
                        WallTextures[S.LowerTexture.ID] = S.LowerTexture.Texture;
                    }
                }

            }


            foreach (Wad.Line L in this.Lines) {
                Wad.Sector LeftSector = L.LeftSector == -1 ? this.Sectors[L.RightSector] : this.Sectors[L.LeftSector];
                Wad.Sector RightSector = L.RightSector == -1 ? this.Sectors[L.LeftSector] : this.Sectors[L.RightSector];
                if (L.RightSide != -1) AddSideAsWall(this.Sides[L.RightSide], ref AllWalls, RightSector, LeftSector, L, false);
                if (L.LeftSide != -1) AddSideAsWall(this.Sides[L.LeftSide], ref AllWalls, LeftSector, RightSector, L, true);
            }

            List<TextureWallGroup> DecodedWalls = new List<TextureWallGroup>();
            foreach (object o in AllWalls.Keys) {
                TextureWallGroup T = new TextureWallGroup(((List<Wall>)AllWalls[o]).ToArray(), (Bitmap)WallTextures[o]);
                if (T.Walls.Length != 0) {
                    DecodedWalls.Add(T);
                }
            }
            this.Walls = DecodedWalls.ToArray();

            this.Floors = SplitUpFloors();
        }

        Random R = new Random();
        private void AddSideAsWall(Wad.Side SideToAdd, ref Hashtable AllWalls, Wad.Sector SectorSideSurrounds, Wad.Sector Adjacent, Wad.Line LineSideIsPartOf, bool OnLeft) {


            Point StartVertex;
            Point EndVertex;


            bool NeedToFlipVertices = OnLeft;

            if (!NeedToFlipVertices) {
                StartVertex = this.Vertices[LineSideIsPartOf.StartVertex];
                EndVertex = this.Vertices[LineSideIsPartOf.EndVertex];                
            } else {
                StartVertex = this.Vertices[LineSideIsPartOf.EndVertex];
                EndVertex = this.Vertices[LineSideIsPartOf.StartVertex];                
            }


            int LightLevel = Math.Max(SectorSideSurrounds.LightLevel, Adjacent.LightLevel);

            if (SideToAdd.MiddleTexture != null) {
                int Y = 0;

                int FloorHeight = !OnLeft ? SectorSideSurrounds.FloorHeight : Adjacent.FloorHeight;
                int CeilingHeight = !OnLeft ? SectorSideSurrounds.CeilingHeight : Adjacent.CeilingHeight;

                if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.LowerUnpegged) != 0) {
                    Y = SideToAdd.MiddleTexture.Height - (CeilingHeight - FloorHeight);
                }


                ((List<Wall>)AllWalls[SideToAdd.MiddleTexture.ID]).Add(
                    new Wall(
                        StartVertex, EndVertex,
                        FloorHeight, CeilingHeight,
                        LightLevel,
                        SideToAdd.MiddleTexture.ID,
                        new Point(SideToAdd.X, SideToAdd.Y + Y),
                        LineSideIsPartOf.Flags, Wall.WallType.Middle
                     )
                 );
            }

            if (Adjacent != null) {

                Wad.Sector HiSector;
                Wad.Sector LoSector;

                if (SideToAdd.UpperTexture != null && !(SectorSideSurrounds.SkyCeiling && Adjacent.SkyCeiling)) {

                    if (Adjacent.CeilingHeight > SectorSideSurrounds.CeilingHeight) {
                        HiSector = Adjacent;
                        LoSector = SectorSideSurrounds;
                    } else {
                        HiSector = SectorSideSurrounds;
                        LoSector = Adjacent;
                    }

                    int Y = 0;
                    if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.UpperUnpegged) == 0) {
                        Y = LoSector.CeilingHeight - HiSector.CeilingHeight;
                    }
                    ((List<Wall>)AllWalls[SideToAdd.UpperTexture.ID]).Add(
                          new Wall(
                              StartVertex, EndVertex,
                              LoSector.CeilingHeight, HiSector.CeilingHeight,
                              LightLevel,
                              SideToAdd.UpperTexture.ID,
                              new Point(SideToAdd.X, SideToAdd.Y + Y),
                              LineSideIsPartOf.Flags, Wall.WallType.Upper
                           )
                       );
                }

                if (SideToAdd.LowerTexture != null && !(SectorSideSurrounds.SkyFloor && Adjacent.SkyFloor)) {

                    if (Adjacent.FloorHeight > SectorSideSurrounds.FloorHeight) {
                        HiSector = Adjacent;
                        LoSector = SectorSideSurrounds;
                    } else {
                        HiSector = SectorSideSurrounds;
                        LoSector = Adjacent;
                    }

                    int Y = 0;

                    if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.LowerUnpegged) != 0) {
                        //Y = HiSector.CeilingHeight
                    }
                    ((List<Wall>)AllWalls[SideToAdd.LowerTexture.ID]).Add(
                          new Wall(
                              StartVertex, EndVertex,
                              LoSector.FloorHeight, HiSector.FloorHeight,
                              LightLevel,
                              SideToAdd.LowerTexture.ID,
                              new Point(SideToAdd.X, SideToAdd.Y + Y),
                              LineSideIsPartOf.Flags, Wall.WallType.Lower
                           )
                       );
                }
            }


            

        }

        private Wad.Sector FindSectorOnOtherSideOfLine(Wad.Sector[] AllSectors, Wad.Sector CurrentSector, Wad.Line SeperatingLine) {
            foreach (Wad.Sector S in AllSectors) {
                if (!S.Equals(CurrentSector) && S.ContainedLineDefs.Contains(SeperatingLine)) {
                    return S;
                }
            }
            return null;
        }

        public class TextureWallGroup {
            public readonly Wall[] Walls;
            public readonly Bitmap Texture;
            public TextureWallGroup(Wall[] Walls, Bitmap Texture) {
                this.Walls = Walls;
                this.Texture = Texture;                
            }
        }

        public class Wall {
            public enum WallType {
                Middle, Upper, Lower
            }
            public readonly Point Start;
            public readonly Point End;
            public readonly int Floor;
            public readonly int Ceiling;
            public readonly int LightLevel;
            public readonly int Texture;
            public readonly Point TextureOffset;
            public readonly Wad.Line.LineFlags Flags;
            public readonly WallType Type;
            public Wall(Point Start, Point End, int Floor, int Ceiling, int LightLevel, int Texture, Point TextureOffset, Wad.Line.LineFlags Flags, WallType Type) {
                this.Start = Start;
                this.End = End;
                this.Floor = Floor;
                this.Ceiling = Ceiling;
                this.LightLevel = LightLevel;
                this.Texture = Texture;
                this.TextureOffset = TextureOffset;
                this.Flags = Flags;
                this.Type = Type;
            }
            public override string ToString() {
                return this.Start.ToString() + ", " + this.End.ToString() + " (" + this.Type.ToString() + ")";
            }

        }
    }
}
