using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using DOOM.WAD;



namespace DOOM.Levels {
    public partial class Level {

        public FloorGroup[] Floors;

        public FloorGroup[] SplitUpFloors() {

            Hashtable FloorGroups = new Hashtable();

            foreach (Wad.Sector S in this.Sectors) {

                //if (S.ID != 1) continue;
                
                //SectorBrush = new SolidBrush(Color.FromArgb(R.Next(0xFF), R.Next(0xFF), R.Next(0xFF)));

                List<Wad.Line> OptimisedLines = new List<Wad.Line>();

                foreach (Wad.Line L in S.ContainedLineDefs) {
                    if (this.Vertices[L.StartVertex].Y != this.Vertices[L.EndVertex].Y
                        && L.StartVertex != L.EndVertex) OptimisedLines.Add(L);
                }

                List<int> YSplits = new List<int>();
                foreach (Wad.Line L in OptimisedLines) {
	                if (!YSplits.Contains(this.Vertices[L.StartVertex].Y)) YSplits.Add(this.Vertices[L.StartVertex].Y);
                    if (!YSplits.Contains(this.Vertices[L.EndVertex].Y)) YSplits.Add(this.Vertices[L.EndVertex].Y);
                }

                YSplits.Sort();
                
                // Now we need to split out the various lines inside this space:
                for (int i = 0; i < YSplits.Count - 1; ++i) {
                    int UpperY = YSplits[i + 0];
                    int LowerY = YSplits[i + 1];

                    List<VerticalWallSpan> YSpanSplitters = new List<VerticalWallSpan>();
                    YSpanSplitters.Clear();
                    foreach (Wad.Line L in OptimisedLines) {

                        // Start and finish vertices:
                        PointF PS = this.Vertices[L.StartVertex];
                        PointF PE = this.Vertices[L.EndVertex];
                        if (Math.Max(PS.Y, PE.Y) < UpperY || Math.Min(PS.Y, PE.Y) > LowerY) continue; // Out of bounds.
                        if (PS.Y == PE.Y) continue; // Useless line

                        // It is technically impossible for a Y coordinate to lie BETWEEN the upper and lower bounds.
                        // However, we SHOULD clip if they go above or below the bounds...

                        PointF LS = PS;
                        PointF LE = PE;
                        
                        ClipPoint(ref PS, LS, LE, UpperY, LowerY);
                        ClipPoint(ref PE, LS, LE, UpperY, LowerY);

                        if (!(L.LeftSector == S.ID && L.RightSector == S.ID)) {
                            if (L.LeftSector == S.ID) {
                                YSpanSplitters.Add(new VerticalWallSpan(PE, PS));
                            }
                            if (L.RightSector == S.ID) {
                                YSpanSplitters.Add(new VerticalWallSpan(PS, PE));
                            }
                        }
                        
                    }
 
                    // YSpanSplitters now contains a sorted list of all the lines that split up a y-span:
                    List<VerticalWallSpan> SortedAndStripped = new List<VerticalWallSpan>();
                    foreach (VerticalWallSpan V in YSpanSplitters) {
                        if (!V.Start.Equals(V.End)) {
                            SortedAndStripped.Add(V);
                        }
                    }
                    SortedAndStripped.Sort();

                    // Remove any redundant walls.
                    /*List<VerticalWallSpan> RemoveRendundantLines = new List<VerticalWallSpan>();
                    int WallNestDepth = 0;
                    for (int j = 0; j < SortedAndStripped.Count; j++) {

                        VerticalWallSpan V = SortedAndStripped[j];

                        if (V.Start.Y <= V.End.Y) {
                            if (WallNestDepth == 0) RemoveRendundantLines.Add(V);
                            ++WallNestDepth;
                        } else {
                            if (WallNestDepth == 1) RemoveRendundantLines.Add(V);
                            --WallNestDepth;
                        }
                        RemoveRendundantLines.Add(V);
                    }

                    SortedAndStripped = RemoveRendundantLines;
                    */

                    for (int j = 0; j < SortedAndStripped.Count - 1; j += 2) {
                        VerticalWallSpan SliceL = SortedAndStripped[j + 0];
                        VerticalWallSpan SliceR = SortedAndStripped[j + 1];

                        PointF Upper1 = (SliceL.Start.Y < SliceL.End.Y) ? SliceL.Start : SliceL.End;
                        PointF Upper2 = (SliceR.Start.Y < SliceR.End.Y) ? SliceR.Start : SliceR.End;
                        PointF Lower1 = (SliceL.Start.Y > SliceL.End.Y) ? SliceL.Start : SliceL.End;
                        PointF Lower2 = (SliceR.Start.Y > SliceR.End.Y) ? SliceR.Start : SliceR.End;

                        PointF TopLeft;
                        PointF TopRight;
                        PointF BtmLeft;
                        PointF BtmRight;

                        if (Upper1.X < Upper2.X) {
                            TopLeft = Upper1;
                            TopRight = Upper2;
                        } else {
                            TopLeft = Upper2;
                            TopRight = Upper1;
                        }


                        if (Lower1.X < Lower2.X) {
                            BtmLeft = Lower1;
                            BtmRight = Lower2;
                        } else {
                            BtmLeft = Lower2;
                            BtmRight = Lower1;
                        
                        }

                        if (!S.SkyFloor) 
                            AddFloorTrapezium(ref FloorGroups, new PointF[] { BtmLeft, BtmRight, TopLeft, TopRight }, S.LightLevel, S.FloorHeight, S.FloorTexture, S);
                        if (!S.SkyCeiling) 
                            AddFloorTrapezium(ref FloorGroups, new PointF[] { TopLeft, TopRight, BtmLeft, BtmRight }, S.LightLevel, S.CeilingHeight, S.CeilingTexture, S);

                        /*G.FillPolygon(SectorBrush, new Point[] { TranslatePoint(TopLeft), TranslatePoint(TopRight), TranslatePoint(BtmRight), TranslatePoint(BtmLeft) });
                        Pen TriangleTracer = Pens.Black;
                        G.DrawPolygon(TriangleTracer, new Point[] { TranslatePoint(TopLeft), TranslatePoint(TopRight), TranslatePoint(BtmRight), TranslatePoint(BtmLeft) });
                        G.DrawLine(TriangleTracer, TranslatePoint(TopLeft), TranslatePoint(BtmRight));*/

                    }
                }
            }

            /*foreach (Wad.Sector S in this.Sectors) {
                foreach (Wad.Line L in S.ContainedLineDefs) {
                    //G.DrawLine(Pens.Black, TranslatePoint(this.Vertices[L.StartVertex]), TranslatePoint(this.Vertices[L.EndVertex]));
                }
            }
            
             */

            //B.Save("test.png");

            bool HasSky = false;
            /*foreach (object O in FloorGroups.Keys) {
                FloorGroup F = (FloorGroup)FloorGroups[O];
                if (F.IsSky) HasSky = true;
            }*/

            FloorGroup[] Return = new FloorGroup[FloorGroups.Keys.Count];
            int k = HasSky ? 1 : 0;
            foreach (object O in FloorGroups.Keys) {
                FloorGroup F = (FloorGroup)FloorGroups[O];
                if (F.IsSky) {
                    Return[0] = F;
                } else {
                    Return[k++] = F;
                }
            }

            int TriangleCount = 0;
            foreach (FloorGroup F in Return) {
                TriangleCount += F.FloorTrapezia.Count;
                
            }
            Console.WriteLine("{0} sectors split into {1} groups: {2} triangles.", this.Sectors.Length, Return.Length, TriangleCount * 2);

            return Return;
        }

        private void AddFloorTrapezium(ref Hashtable FloorGroups, PointF[] Points, int LightLevel, int Height, Wad.Lump Flat, Wad.Sector Sector) {
            string TextureName = Flat.Name;
            object FindFloorGroup = FloorGroups[TextureName];
            FloorGroup F;
            if (FindFloorGroup == null) {
                F = new FloorGroup(Flat);
                FloorGroups[TextureName] = F;
            } else {
                F = (FloorGroup)FindFloorGroup;
            }
            F.FloorTrapezia.Add(new FloorTrapezium(Points[0], Points[1], Points[2], Points[3], Height, LightLevel, Sector));
        }

        private class VerticalWallSpan : IComparable<VerticalWallSpan> {
            public readonly PointF Start;
            public readonly PointF End;
            public int CompareTo(VerticalWallSpan Other) {
                if (Other.Equals(this)) {
                    return 0;
                } else {
                    float ThisX = this.Start.X + this.End.X;
                    float OtherX = Other.Start.X + Other.End.X;
                    return ThisX.CompareTo(OtherX);
                }
            }
            public VerticalWallSpan(PointF StartX, PointF EndX) {
                this.Start = StartX;
                this.End = EndX;
            }
        }

        public class FloorGroup {
            public List<FloorTrapezium> FloorTrapezia;
            public readonly Wad.Lump Texture;
            public readonly bool IsSky;
            public FloorGroup(Wad.Lump Texture) {
                this.Texture = Texture;
                this.FloorTrapezia = new List<FloorTrapezium>();
                this.IsSky = Texture.Name == "F_SKY1";
            }
        }

        public class FloorTrapezium {
            public readonly PointF TopLeft;
            public readonly PointF TopRight;
            public readonly PointF BtmLeft;
            public readonly PointF BtmRight;
            public readonly int Height;
            public readonly int LightLevel;
            public readonly Wad.Sector Sector;
            public FloorTrapezium(PointF TopLeft, PointF TopRight, PointF BtmLeft, PointF BtmRight, int Height, int LightLevel, Wad.Sector Sector) {
                this.TopLeft = TopLeft;
                this.TopRight = TopRight;
                this.BtmLeft = BtmLeft;
                this.BtmRight = BtmRight;

                this.Height = Height;
                this.LightLevel = LightLevel;

                this.Sector = Sector;
            }
        }

        private void ClipPoint(ref PointF Pt, PointF LineStart, PointF LineEnd, int UpperY, int LowerY) {
            bool Changed = false;
            if (Pt.Y < UpperY) {
                Pt.Y = UpperY;
                Changed = true;
            }
            if (Pt.Y > LowerY) {
                Pt.Y = LowerY;
                Changed = true;
            }
            if (Changed) {
                Pt.X = (float)Math.Round((((decimal)Pt.Y - (decimal)LineStart.Y) * ((decimal)LineEnd.X - (decimal)LineStart.X)) / ((decimal)LineEnd.Y - (decimal)LineStart.Y) + (decimal)LineStart.X);
            }
        }

        Point Offset;
        private Point TranslatePoint(Point P) {
            return new Point((P.X + Offset.X) / 4, (P.Y + Offset.Y) / 4);
        }

    }
}
