using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using DOOM.WAD;



namespace DOOM.Levels {
    public partial class Level {

        public Point[] Vertices;
        public Wad.Line[] Lines;
        public Wad.Side[] Sides;
        public Wad.Sector[] Sectors;
        public TextureWallGroup[] Walls;
        public Wad.Segment[] Segments;
        public Wad.SubSector[] SubSectors;
        public Wad.Thing[] Things;
        public Wad.BspNode[] Nodes;
        public readonly string Name = "";
        public readonly int LevelIndex;
        public Wad WadFile;


        public Wad.Sector GetSectorAtPoint(Point Pt) {
            foreach (Wad.Sector S in this.Sectors) {
                bool IsTheOne = true;
                foreach (Wad.Segment Seg in S.ContainedSegments) {
                    
                    Vector2 A = new Vector2(this.Vertices[Seg.StartVertex].X, this.Vertices[Seg.StartVertex].Y) - new Vector2(Pt.X, Pt.Y);
                    Vector2 B = new Vector2(this.Vertices[Seg.EndVertex].X, this.Vertices[Seg.EndVertex].Y) - new Vector2(Pt.X, Pt.Y);

                    if (Seg.OppositeDirectionToLine) {
                        Vector2 Swap = A;
                        A = B;
                        B = Swap;
                    }

                    if (Vector2.Ccw(A, B) > 0) {
                        IsTheOne = false;
                        break;
                    }
                }
                
                        
                if (IsTheOne) return S;
            }
            
            return null;
        }

        public Level(Wad WadFile, string LevelName) {
            this.WadFile = WadFile;

            int FindLevel = 0;
            LevelIndex = 0;
            for (; FindLevel < WadFile.LumpCount; ++FindLevel) {
                if (WadFile.Lumps[FindLevel].Type == Wad.LumpType.Marker) {
                    ++LevelIndex;
                    if (WadFile.Lumps[FindLevel].Name.ToUpper() == LevelName.ToUpper()) {
                        goto FoundLevel;
                    }
                }                
            }
            FoundLevel:

            if (FindLevel == WadFile.LumpCount) throw new Exception("Level not found.");

            this.Name = LevelName;

            Wad.Lump VertexLump = null;
            Wad.Lump LineDefLump = null;
            Wad.Lump SideDefLump = null;
            Wad.Lump SectorLump = null;
            Wad.Lump SubSectorLump = null;
            Wad.Lump SegmentLump = null;
            Wad.Lump ThingLump = null;
            Wad.Lump NodeLump = null;

            for (int i = FindLevel + 1; i < WadFile.LumpCount; ++i) {
                Wad.Lump DetectLump = WadFile.Lumps[i];
                if (DetectLump.Type == Wad.LumpType.Marker) break;
                switch (DetectLump.Type) {
                    case Wad.LumpType.VertexList:
                        if (VertexLump == null) VertexLump = DetectLump;
                        break;
                    case Wad.LumpType.LineDefinitions:
                        if (LineDefLump == null) LineDefLump = DetectLump;
                        break;
                    case Wad.LumpType.SideDefinitions:
                        if (SideDefLump == null) SideDefLump = DetectLump;
                        break;
                    case Wad.LumpType.SectorList:
                        if (SectorLump == null) SectorLump = DetectLump;
                        break;
                    case Wad.LumpType.SubsectorList:
                        if (SubSectorLump == null) SubSectorLump = DetectLump;
                        break;
                    case Wad.LumpType.SegmentList:
                        if (SegmentLump == null) SegmentLump = DetectLump;
                        break;
                    case Wad.LumpType.Things:
                        if (ThingLump == null) ThingLump = DetectLump;
                        break;
                    case Wad.LumpType.NodeList:
                        if (NodeLump == null) NodeLump = DetectLump;
                        break;
                }
            }

            if (VertexLump == null) throw new Exception("Could not find vertex lump.");
            if (LineDefLump == null) throw new Exception("Could not find linedef lump.");
            if (SideDefLump == null) throw new Exception("Could not find sidedef lump.");
            if (SectorLump == null) throw new Exception("Could not find sector lump.");
            if (SegmentLump == null) throw new Exception("Could not find segment lump.");
            if (SubSectorLump == null) throw new Exception("Could not find subsector lump.");
            if (ThingLump == null) throw new Exception("Could not find things lump.");
            if (NodeLump == null) throw new Exception("Could not find nodes lump.");


            this.Vertices = (Point[])VertexLump.DecodedData;
            this.Lines = (Wad.Line[])LineDefLump.DecodedData;
            this.Sides = (Wad.Side[])SideDefLump.DecodedData;
            this.Sectors = (Wad.Sector[])SectorLump.DecodedData;
            this.Segments = (Wad.Segment[])SegmentLump.DecodedData;
            this.SubSectors = (Wad.SubSector[])SubSectorLump.DecodedData;
            this.Things = (Wad.Thing[])ThingLump.DecodedData;
            this.Nodes = (Wad.BspNode[])NodeLump.DecodedData;

            // Index the line definitions

            Console.WriteLine("Indexing line definitions by sector.");
            foreach (Wad.Sector S in Sectors) {
                S.IndexLineDefs(this.Lines, this.Sides, this.Segments);
                S.AdjacentSectors = new List<Wad.Sector>(); // Set this up for the next thingummy;
            }
            Console.WriteLine("Linking adjacent sectors.");
            foreach (Wad.Line L in Lines) {
                if (L.LeftSector != -1 && L.RightSector != -1) {
                    Wad.Sector LS = Sectors[L.LeftSector];
                    Wad.Sector RS = Sectors[L.RightSector];
                    if (!LS.AdjacentSectors.Contains(RS)) LS.AdjacentSectors.Add(RS);
                    if (!RS.AdjacentSectors.Contains(LS)) RS.AdjacentSectors.Add(LS);
                }
            }

            
            ////////////////////////////////////////////////////////////////////////////////////////////////////

            /*
            int MinX = 9999999;
            int MaxX = -9999999;
            int MinY = 9999999;
            int MaxY = -9999999;
            foreach (Point P in this.Vertices) {
                MinX = Math.Min(MinX, P.X);
                MaxX = Math.Max(MaxX, P.X);
                MinY = Math.Min(MinY, P.Y);
                MaxY = Math.Max(MaxY, P.Y);
            }
            Offset = new Point(-MinX, -MinY);

            Bitmap B = new Bitmap((MaxX - MinX) / 4 + 1, (MaxY - MinY) / 4 + 1);
            Graphics G = Graphics.FromImage(B);
                        
            Random R = new Random();

            //this.Nodes[0]

            foreach (Wad.SubSector SS in this.SubSectors) {
                List<Point> PointsInSubsector = new List<Point>();


                for (int i = SS.SegIndex; i < SS.SegIndex + SS.SegCount; i++) {
                    Wad.Segment Seg = this.Segments[i];

                    Wad.Sector ContainingSector = null;
                    if (Seg.OppositeDirectionToLine) {
                        ContainingSector = this.Sectors[this.Lines[Seg.Line].LeftSector];
                    } else {
                        ContainingSector = this.Sectors[this.Lines[Seg.Line].RightSector];
                    }

                    if (ContainingSector.ID != 1) continue;


                    Point EndVertex = this.Vertices[Seg.OppositeDirectionToLine ? Seg.StartVertex : Seg.EndVertex];
                    Point StartVertex = this.Vertices[Seg.OppositeDirectionToLine ? Seg.EndVertex : Seg.StartVertex];

                    G.DrawLine(Pens.Black, TranslatePoint(StartVertex), TranslatePoint(EndVertex));
                    G.DrawRectangle(Pens.Red, new Rectangle(TranslatePoint(StartVertex).X - 3, TranslatePoint(StartVertex).Y - 3, 6, 6));
                    PointsInSubsector.Add(TranslatePoint(StartVertex));
                    PointsInSubsector.Add(TranslatePoint(EndVertex));
                }
                if (PointsInSubsector.Count != 0) {
                    Color SectorColour = Color.FromArgb(R.Next(0xFF), R.Next(0xFF), R.Next(0xFF));

                    //G.FillPolygon(new SolidBrush(SectorColour), PointsInSubsector.ToArray());

                    for (int i = 0; i < PointsInSubsector.Count - 1; i++) {
                    //    G.FillPolygon(new SolidBrush(SectorColour), new Point[] { PointsInSubsector[0], PointsInSubsector[i + 0], PointsInSubsector[i + 1] });
                    }
                }
                
            }
            B.Save("test.png");
            

            //*/

            ////////////////////////////////////////////////////////////////////////////////////////////////////

            // Now we need to encode the level into a series of vertex buffers,
            // one per texture.
            
            Hashtable AllWalls = new Hashtable();
            Hashtable WallTextures = new Hashtable();


            foreach (Wad.Side S in Sides) {

                if (S.MiddleTexture != null) {
                    object FindTexture = AllWalls[S.MiddleTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.MiddleTexture.ID] = new List<Wall>();
                        WallTextures[S.MiddleTexture.ID] = S.MiddleTexture;
                    }
                }
                if (S.UpperTexture != null) {
                    object FindTexture = AllWalls[S.UpperTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.UpperTexture.ID] = new List<Wall>();
                        WallTextures[S.UpperTexture.ID] = S.UpperTexture;
                    }
                }
                if (S.LowerTexture != null) {
                    object FindTexture = AllWalls[S.LowerTexture.ID];
                    if (FindTexture == null) {
                        AllWalls[S.LowerTexture.ID] = new List<Wall>();
                        WallTextures[S.LowerTexture.ID] = S.LowerTexture;
                    }
                }

            }


            foreach (Wad.Line L in this.Lines) {
                Wad.Sector LeftSector = L.LeftSector == -1 ? this.Sectors[L.RightSector] : this.Sectors[L.LeftSector];
                Wad.Sector RightSector = L.RightSector == -1 ? this.Sectors[L.LeftSector] : this.Sectors[L.RightSector];
                if (L.RightSide != -1) {
                    AddSideAsWall(this.Sides[L.RightSide], ref AllWalls, RightSector, LeftSector, L, false);
                    if ((L.Flags & Wad.Line.LineFlags.TwoSided) != 0) {
                        //AddSideAsWall(this.Sides[L.RightSide], ref AllWalls, RightSector, LeftSector, L, true);
                    }
                }
                if (L.LeftSide != -1) {
                    AddSideAsWall(this.Sides[L.LeftSide], ref AllWalls, LeftSector, RightSector, L, true);
                    if ((L.Flags & Wad.Line.LineFlags.TwoSided) != 0) {
                        //AddSideAsWall(this.Sides[L.LeftSide], ref AllWalls, LeftSector, RightSector, L, false);
                    }
                }

            }

            List<TextureWallGroup> DecodedWalls = new List<TextureWallGroup>();
            foreach (object o in AllWalls.Keys) {
                TextureWallGroup T = new TextureWallGroup(((List<Wall>)AllWalls[o]).ToArray(), (Wad.WallTexture)WallTextures[o]);
                if (T.Walls.Length != 0) {
                    DecodedWalls.Add(T);
                }
            }
            this.Walls = DecodedWalls.ToArray();

            this.Floors = SplitUpFloors();

            

        }

        Random R = new Random();
        private void AddSideAsWall(Wad.Side SideToAdd, ref Hashtable AllWalls, Wad.Sector SectorSideSurrounds, Wad.Sector Adjacent, Wad.Line LineSideIsPartOf, bool OnLeft) {

            Point StartVertex;
            Point EndVertex;


            bool NeedToFlipVertices = OnLeft;

            if (!NeedToFlipVertices) {
                StartVertex = this.Vertices[LineSideIsPartOf.StartVertex];
                EndVertex = this.Vertices[LineSideIsPartOf.EndVertex];                
            } else {
                StartVertex = this.Vertices[LineSideIsPartOf.EndVertex];
                EndVertex = this.Vertices[LineSideIsPartOf.StartVertex];                
            }

            if (OnLeft) {
                Wad.Sector Swap = Adjacent;
                Adjacent = SectorSideSurrounds;
                SectorSideSurrounds = Swap;
            }

            int MaxC = Math.Max(SectorSideSurrounds.CeilingHeight, Adjacent.CeilingHeight);
            int MinC = Math.Min(SectorSideSurrounds.CeilingHeight, Adjacent.CeilingHeight);
            int MaxF = Math.Max(SectorSideSurrounds.FloorHeight, Adjacent.FloorHeight);
            int MinF = Math.Min(SectorSideSurrounds.FloorHeight, Adjacent.FloorHeight);

              
            int LightLevel = Math.Min(SectorSideSurrounds.LightLevel, Adjacent.LightLevel);
            LightLevel = SectorSideSurrounds.LightLevel;

            if (SideToAdd.MiddleTexture != null) {
                int Y = 0;

                //int FloorHeight = !OnLeft ? SectorSideSurrounds.FloorHeight : Adjacent.FloorHeight;
                //int CeilingHeight = !OnLeft ? SectorSideSurrounds.CeilingHeight : Adjacent.CeilingHeight;

                if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.LowerUnpegged) != 0) {
                    Y = SideToAdd.MiddleTexture.Height - (MinC - MaxF);
                }


                ((List<Wall>)AllWalls[SideToAdd.MiddleTexture.ID]).Add(
                    new Wall(
                        StartVertex, EndVertex,
                        MaxF, MinC,
                        LightLevel,
                        SideToAdd.MiddleTexture.ID,
                        new Point(SideToAdd.X, SideToAdd.Y + Y),
                        LineSideIsPartOf.Flags, Wall.WallType.Middle,
                        SectorSideSurrounds,
                        LineSideIsPartOf.Type == Wad.Line.LineType.ScrollingWall
                     )
                 );
            }

            if (Adjacent != null) {


                if (SideToAdd.UpperTexture != null && !(SectorSideSurrounds.SkyCeiling && Adjacent.SkyCeiling)) {

                    int Y = 0;
                    if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.UpperUnpegged) == 0) {
                        Y = MinC - MaxC;
                    }
                    ((List<Wall>)AllWalls[SideToAdd.UpperTexture.ID]).Add(
                          new Wall(
                              StartVertex, EndVertex,
                              Math.Min(MaxC, MinC), Math.Max(MaxC, MinC),
                              LightLevel,
                              SideToAdd.UpperTexture.ID,
                              new Point(SideToAdd.X, SideToAdd.Y + Y),
                              LineSideIsPartOf.Flags, Wall.WallType.Upper,
                              SectorSideSurrounds,
                              LineSideIsPartOf.Type == Wad.Line.LineType.ScrollingWall
                           )
                       );
                }

                if (SideToAdd.LowerTexture != null && !(SectorSideSurrounds.SkyFloor && Adjacent.SkyFloor)) {

       

                    int Y = 0;

                    if ((LineSideIsPartOf.Flags & Wad.Line.LineFlags.LowerUnpegged) != 0) {
                        Y = MaxC - MaxF;
                    }
                    ((List<Wall>)AllWalls[SideToAdd.LowerTexture.ID]).Add(
                          new Wall(
                              StartVertex, EndVertex,
                              Math.Min(MaxF, MinF), Math.Max(MaxF, MinF),
                              LightLevel,
                              SideToAdd.LowerTexture.ID,
                              new Point(SideToAdd.X, SideToAdd.Y + Y),
                              LineSideIsPartOf.Flags, Wall.WallType.Lower,
                              SectorSideSurrounds,
                              LineSideIsPartOf.Type == Wad.Line.LineType.ScrollingWall
                           )
                       );
                }
            }


            

        }

        private Wad.Sector FindSectorOnOtherSideOfLine(Wad.Sector[] AllSectors, Wad.Sector CurrentSector, Wad.Line SeperatingLine) {
            foreach (Wad.Sector S in AllSectors) {
                if (!S.Equals(CurrentSector) && S.ContainedLineDefs.Contains(SeperatingLine)) {
                    return S;
                }
            }
            return null;
        }

        public class TextureWallGroup {
            public readonly Wall[] Walls;
            public readonly Wad.WallTexture Texture;
            public TextureWallGroup(Wall[] Walls, Wad.WallTexture Texture) {
                this.Walls = Walls;
                this.Texture = Texture;                
            }
        }

        public class Wall {
            public enum WallType {
                Middle, Upper, Lower
            }
            public readonly Point Start;
            public readonly Point End;
            public readonly int Floor;
            public readonly int Ceiling;
            public readonly int LightLevel;
            public readonly int Texture;
            public readonly Point TextureOffset;
            public readonly Wad.Line.LineFlags Flags;
            public readonly WallType Type;
            public readonly Wad.Sector Sector;
            public readonly bool Scrolling;
            public Wall(Point Start, Point End, int Floor, int Ceiling, int LightLevel, int Texture, Point TextureOffset, Wad.Line.LineFlags Flags, WallType Type, Wad.Sector Sector, bool Scrolling) {
                this.Start = Start;
                this.End = End;
                this.Floor = Floor;
                this.Ceiling = Ceiling;
                this.LightLevel = LightLevel;
                this.Texture = Texture;
                this.TextureOffset = TextureOffset;
                this.Flags = Flags;
                this.Type = Type;
                this.Sector = Sector;
                this.Scrolling = Scrolling;
            }
            public override string ToString() {
                return this.Start.ToString() + ", " + this.End.ToString() + " (" + this.Type.ToString() + ")";
            }

        }
    }
}
