using System;
using System.Windows.Forms;
using DOOM.WAD;
using DOOM.Levels;
using DOOM.Music;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace DOOM {
    static class Program {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args) {

            if (args.Length != 2) {
                Console.WriteLine("Usage: DOOM <wad> <level>");
                return;
            }


            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            using (DOOM D = new DOOM()) {

                if (!D.InitialiseDevice()) {
                    return;
                }


                MidiDevice MidiDev = null;
                try {
                    MidiDev = new MidiDevice();
                } catch (Exception ex) {
                    Console.WriteLine(ex.Message);
                }

                D.WadFile = new Wad(args[0]);
                D.CurrentLevel = new Level(D.WadFile, args[1]);
                D.InitialiseWallBuffers();
                D.InitialiseFloorBuffers();
                D.InitialiseThingTextures();
                D.InitialiseSkyBuffer();
                D.StartUpdateEnvironment();

                int SongIndex = D.CurrentLevel.LevelIndex;
                Wad.Lump Music = null;
                foreach (Wad.Lump L in D.CurrentLevel.WadFile.Lumps) {
                    if (L.Type == Wad.LumpType.Song && L.Name == "D_" + D.CurrentLevel.Name) {
                        Music = L;
                        break;
                    }
                }
                if (Music == null) {
                    foreach (Wad.Lump L in D.CurrentLevel.WadFile.Lumps) {

                        if (L.Type == Wad.LumpType.Song) {
                            --SongIndex;
                            if (SongIndex == 0) {
                                Music = L;
                                break;
                            }
                        }
                    }
                }

                if (Music != null) {
                    MusPlayer MP = new MusPlayer(MidiDev, Music.Data);
                }

                foreach (Wad.Thing T in D.CurrentLevel.Things) {
                    if (T.Type == Wad.Thing.ThingType.Player1Start) {
                        D.PlayerPosition = new Vector3(T.Position.X, T.Position.Y, 0.0f);
                        D.PlayerView = (((float)T.Angle) * (float)Math.PI) / 180.0f - (float)Math.PI / 2.0f;
                    }
                }
                D.Show();
                while (D.Created) {
                    D.UpdateEnvironment();
                    D.Render();
                    Application.DoEvents();
                }

            }


        }
    }
}