using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Collections.Generic;
using DOOM.WAD;
using DOOM.Levels;

namespace DOOM {
    public partial class DOOM {

        //VertexBuffer[] FloorBuffer = null;
        //Texture[] FloorTextures = null;
        TextureAnimation[] Floors;

        void AddFlatToAnimation(ref TextureAnimation ToAddTo, Wad.Lump Texture) {
            MemoryStream M = new MemoryStream();
            Bitmap TextureToUse = (Bitmap)Texture.DecodedData;
            TextureToUse.Save(M, ImageFormat.Png);
            M.Seek(0, SeekOrigin.Begin);
            ToAddTo.TextureList.Add(TextureLoader.FromStream(RenderDevice, M));
        }

        public void InitialiseFloorBuffers() {
            Floors = new TextureAnimation[CurrentLevel.Floors.Length];

            for (int i = 0; i < CurrentLevel.Floors.Length; ++i) {

                Floors[i] = new TextureAnimation();

                Wad.AnimatedFlat AnimatedFlatDetails = null;
                foreach (Wad.AnimatedFlat AF in CurrentLevel.WadFile.AnimatedFlats) {
                    if (AF.AllTextureNames.Contains(CurrentLevel.Floors[i].Texture.Name)) {
                        AnimatedFlatDetails = AF;
                        break;
                    }
                }
                if (AnimatedFlatDetails == null) {
                    AddFlatToAnimation(ref Floors[i], CurrentLevel.Floors[i].Texture);
                } else {
                    foreach (Wad.Lump L in AnimatedFlatDetails.AllTextures) {
                        AddFlatToAnimation(ref Floors[i], L);
                    }
                }

                Dictionary<Wad.Sector, List<Level.FloorTrapezium>> SortBySector = new Dictionary<Wad.Sector, List<Level.FloorTrapezium>>();
                foreach (Level.FloorTrapezium T in CurrentLevel.Floors[i].FloorTrapezia) {
                    List<Level.FloorTrapezium> SectorFloorList;
                    if (!SortBySector.TryGetValue(T.Sector, out SectorFloorList)) {
                        SortBySector[T.Sector] = new List<Level.FloorTrapezium>();
                        SectorFloorList = SortBySector[T.Sector];
                    }
                    SectorFloorList.Add(T);
                }

                Floors[i].Geometry = new VertexBuffer(typeof(CustomVertex.PositionColoredTextured), 6 * CurrentLevel.Floors[i].FloorTrapezia.Count, RenderDevice, Usage.None, CustomVertex.PositionColoredTextured.Format, Pool.Default);
                CustomVertex.PositionColoredTextured[] Vertices = new CustomVertex.PositionColoredTextured[6 * CurrentLevel.Floors[i].FloorTrapezia.Count];
                int VertexIndex = 0;

                foreach (KeyValuePair<Wad.Sector, List<Level.FloorTrapezium>> KVP in SortBySector) {


                    List<SectorComponent> SectorComponentToAddTo;
                    if (!SectorToVerticesLinkup.TryGetValue(KVP.Key, out SectorComponentToAddTo)) {
                        SectorComponentToAddTo = new List<SectorComponent>();
                        SectorToVerticesLinkup[KVP.Key] = SectorComponentToAddTo;
                    }

                    int InitialVertexIndex = VertexIndex;

                    foreach (Level.FloorTrapezium T in KVP.Value) {

                        float FudgeTop = +0.0f;
                        float FudgeBtm = -0.0f;
                        float FudgeLft = -0.0f;
                        float FudgeRgt = +0.0f;

                        if (T.TopLeft.Y < T.BtmLeft.Y) {
                            FudgeTop *= -1.0f;
                            FudgeBtm *= -1.0f;
                        }

                        if (T.TopLeft.X > T.TopRight.X) {
                            FudgeLft *= -1.0f;
                            FudgeRgt *= -1.0f;
                        }

                        for (int j = 0; j < 6; j++) {
                            Vertices[VertexIndex + j].Z = T.Height;
                            Vertices[VertexIndex + j].Color = T.LightLevel + (T.LightLevel << 8) + (T.LightLevel << 16);
                        }

                        Vertices[VertexIndex + 0].X = T.TopLeft.X + FudgeLft;
                        Vertices[VertexIndex + 0].Y = T.TopLeft.Y + FudgeTop;

                        Vertices[VertexIndex + 1].X = T.TopRight.X + FudgeRgt;
                        Vertices[VertexIndex + 1].Y = T.TopRight.Y + FudgeTop;

                        Vertices[VertexIndex + 2].X = T.BtmRight.X + FudgeRgt;
                        Vertices[VertexIndex + 2].Y = T.BtmRight.Y + FudgeBtm;

                        Vertices[VertexIndex + 3].X = T.TopLeft.X + FudgeLft;
                        Vertices[VertexIndex + 3].Y = T.TopLeft.Y + FudgeTop;

                        Vertices[VertexIndex + 4].X = T.BtmRight.X + FudgeRgt;
                        Vertices[VertexIndex + 4].Y = T.BtmRight.Y + FudgeBtm;

                        Vertices[VertexIndex + 5].X = T.BtmLeft.X + FudgeLft;
                        Vertices[VertexIndex + 5].Y = T.BtmLeft.Y + FudgeBtm;

                        for (int j = 0; j < 6; j++) {
                            Vertices[VertexIndex + j].Z = T.Height;
                            Vertices[VertexIndex + j].Color = T.LightLevel + (T.LightLevel << 8) + (T.LightLevel << 16);
                            Vertices[VertexIndex + j].Tu = Vertices[VertexIndex + j].X / 64.0f;
                            Vertices[VertexIndex + j].Tv = 1.0f - Vertices[VertexIndex + j].Y / 64.0f;
                        }
                        // Move on...
                        VertexIndex += 6;
                    }

                    int NumberOfVertices = VertexIndex - InitialVertexIndex;
                    int Size = NumberOfVertices * CustomVertex.PositionColoredTextured.StrideSize;
                    CustomVertex.PositionColoredTextured[] SectorComponentVertices = new CustomVertex.PositionColoredTextured[NumberOfVertices];
                    for (int j = 0; j < NumberOfVertices; ++j) {
                        SectorComponentVertices[j] = Vertices[j + InitialVertexIndex];
                    }
                    SectorComponentToAddTo.Add(new SectorComponent(Floors[i].Geometry, InitialVertexIndex * CustomVertex.PositionColoredTextured.StrideSize, Size, SectorComponentVertices));

                }
                GraphicsStream Writer = Floors[i].Geometry.Lock(0, 0, 0);
                Writer.Write(Vertices);
                Floors[i].Geometry.Unlock();
                Floors[i].GeometryCount = VertexIndex / 3;
            }

        }

        

        public void DrawFloors() {
            RenderDevice.VertexFormat = CustomVertex.PositionColoredTextured.Format;
            
            /*RenderDevice.RenderState.ColorWriteEnable = 0;
            RenderDevice.SetStreamSource(0, FloorBuffer[0], 0);
            RenderDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, CurrentLevel.Floors[0].FloorTrapezia.Count * 2);
            RenderDevice.RenderState.ColorWriteEnable =  ColorWriteEnable.RedGreenBlueAlpha;*/


            foreach (TextureAnimation T in Floors) {
                RenderDevice.SetTexture(0, T.TextureList[T.TextureIndex]);
                RenderDevice.SetStreamSource(0, T.Geometry, 0);
                RenderDevice.DrawPrimitives(PrimitiveType.TriangleList, 0, T.GeometryCount);
            }

        }

        
    }
}
