using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Drawing;

using DOOM.WAD;

namespace DOOM {
    public partial class DOOM {

        public Vector3 PlayerPosition;
        public float PlayerView;

        DateTime LastFrame = DateTime.Now;
        DateTime LastFps = DateTime.Now;


        int FramesDrawn = 0;

        public void Render() {
            if (RenderDevice == null) return;

            DateTime ThisFrame = DateTime.Now;
            TimeSpan TimePassed = ThisFrame - LastFrame;
            TimeSpan SinceLastFps = ThisFrame - LastFps;
            LastFrame = ThisFrame;

            if (SinceLastFps.TotalSeconds >= 1.0d) {
                //this.Text = ((double)((double)FramesDrawn / SinceLastFps.TotalSeconds)).ToString("F2") + " FPS";
                FramesDrawn = 0;
                LastFps = ThisFrame;
            }

            /*Wad.Sector CurrentSector = CurrentLevel.GetSectorAtPoint(new Point((int)PlayerPosition.X, (int)PlayerPosition.Y));
            if (CurrentSector != null) {
                this.Text = CurrentSector.ID.ToString();
            } else {
                this.Text = "???";
            }*/

            PlayerView += (float)TimePassed.TotalSeconds * LookSpeed;

            Vector3 OldPlayerPosition = PlayerPosition;

            PlayerPosition += (float)TimePassed.TotalMilliseconds * MoveSpeed * new Vector3((float)Math.Sin(PlayerView), (float)Math.Cos(PlayerView), 0.0f);
            PlayerPosition += (float)TimePassed.TotalMilliseconds * HeightSpeed * new Vector3(0.0f, 0.0f, 1.0f);

            /*foreach (Wad.Line L in CurrentLevel.Lines) {
                if ((L.Flags & Wad.Line.LineFlags.Impassable) != 0 || L.LeftSector == -1) {
                    // It's solid.
                    // Have we crossed it?

                    Point VertS = CurrentLevel.Vertices[L.StartVertex];
                    Point VertE = CurrentLevel.Vertices[L.EndVertex];

                    Vector2 OldToS = new Vector2(OldPlayerPosition.X - VertS.X, OldPlayerPosition.Y - VertS.Y);
                    Vector2 OldToE = new Vector2(OldPlayerPosition.X - VertE.X, OldPlayerPosition.Y - VertE.Y);

                    Vector2 NewToS = new Vector2(PlayerPosition.X - VertS.X, PlayerPosition.Y - VertS.Y);
                    Vector2 NewToE = new Vector2(PlayerPosition.X - VertE.X, PlayerPosition.Y - VertE.Y);

                    bool OldSide = Vector2.Ccw(OldToS, OldToE) > 0 ? true : false;
                    bool NewSide = Vector2.Ccw(NewToS, NewToE) > 0 ? true : false;
                    if (OldSide != NewSide) {
                        // Calculate the point of the intersection:
                        //Console.WriteLine("Crossed linedef " + L.ID);
                    }
                    
                }
            }*/

            RenderDevice.Clear(ClearFlags.ZBuffer, System.Drawing.Color.Black, 1.0f, 0);


            RenderDevice.Transform.Projection = Matrix.PerspectiveFovRH((float)Math.PI / 3.5f, 1.75f, 0.1f, 20000.0f);

            RenderDevice.BeginScene();

            RenderDevice.Transform.View = Matrix.LookAtRH(
                Vector3.Empty,
                new Vector3((float)Math.Sin(PlayerView), (float)Math.Cos(PlayerView), 0.0f),
                new Vector3(0.0f, 0.0f, 1.0f)
            );

            DrawSky();

            RenderDevice.Transform.View = Matrix.LookAtRH(
                PlayerPosition,
                PlayerPosition + new Vector3((float)Math.Sin(PlayerView), (float)Math.Cos(PlayerView), 0.0f),
                new Vector3(0.0f, 0.0f, 1.0f)
            );

            DrawFloors();
            DrawWalls();
            

            RenderDevice.EndScene();

            //DrawThings();

            RenderDevice.Present();

            ++FramesDrawn;

        }
    }
}
