using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.Collections.Generic;
using System.IO;

using DOOM.WAD;
using DOOM.Levels;

namespace DOOM {
    public partial class DOOM {

        public Texture SkyTexture;
        public VertexBuffer SkyBuffer;
        public VertexBuffer[] SkyBufferCap;

        const int NumSkySlices = 16;

        public void InitialiseSkyBuffer() {

            const float TopOfSky = 50;
            const float BottomOfSky = -30;

            SkyBuffer = new VertexBuffer(typeof(CustomVertex.PositionTextured), (NumSkySlices + 2) * 2, RenderDevice, Usage.None, CustomVertex.PositionTextured.Format, Pool.Default);

            SkyBufferCap = new VertexBuffer[2];
            for (int i = 0; i < 2; i++) {
                SkyBufferCap[i] = new VertexBuffer(typeof(CustomVertex.PositionTextured), NumSkySlices + 2, RenderDevice, Usage.None, CustomVertex.PositionTextured.Format, Pool.Default);
            }

            CustomVertex.PositionTextured[] Verts = new CustomVertex.PositionTextured[(NumSkySlices + 2) * 2];

            CustomVertex.PositionTextured[] CapA = new CustomVertex.PositionTextured[NumSkySlices + 2];
            CustomVertex.PositionTextured[] CapB = new CustomVertex.PositionTextured[NumSkySlices + 2];


            string SkyName = "SKY1";
            if (CurrentLevel.Name.Length == 4 && CurrentLevel.Name[0] == 'E' && CurrentLevel.Name[2] == 'M') {
                SkyName = "SKY" + CurrentLevel.Name[1];
            } else if (CurrentLevel.Name.Length == 5 && CurrentLevel.Name.StartsWith("MAP")) {
                int LevelNumber = Convert.ToInt16(CurrentLevel.Name.Substring(3));
                if (LevelNumber >= 12) SkyName = "SKY2";
                if (LevelNumber >= 21) SkyName = "SKY3";
            }

            object SkyT = this.CurrentLevel.WadFile.Textures[SkyName];
            if (SkyT == null) throw new Exception("No sky texture in WAD!");
            Wad.WallTexture GetSky = (Wad.WallTexture)SkyT;

            MemoryStream M = new MemoryStream();
            GetSky.Texture.Save(M, ImageFormat.Png);
            M.Seek(0, SeekOrigin.Begin);
            SkyTexture = TextureLoader.FromStream(RenderDevice, M);
            

            float SkyWidth =  (1024.0f / (float)NumSkySlices) / (float)GetSky.Width;

            for (int i = 0; i <= NumSkySlices; ++i) {
                double Angle = (Math.PI * 2 * (double)i) / (double)NumSkySlices;
                for (int j = 0; j < 2; j++) {
                    Verts[i * 2 + j].X = (float)(100.0d * Math.Sin(Angle));
                    Verts[i * 2 + j].Y = (float)(100.0d * Math.Cos(Angle));
                    Verts[i * 2 + j].Tu = SkyWidth * (float)i;
                }

                Verts[i * 2 + 1].Z = TopOfSky;
                Verts[i * 2 + 1].Tv = 0.0f;

                Verts[i * 2 + 0].Z = BottomOfSky;
                Verts[i * 2 + 0].Tv = 1.0f;

                CapA[CapA.Length - 1 - i].X = Verts[i * 2].X;
                CapA[CapA.Length - 1 - i].Y = Verts[i * 2].Y;
                CapA[i + 1].Z = TopOfSky - 1;

                CapA[i + 1].Tu = 0.0f; // SkyWidth * (float)i;
                CapA[i + 1].Tv = 0.0f;


                CapB[i + 1].X = Verts[i * 2].X;
                CapB[i + 1].Y = Verts[i * 2].Y;
                CapB[i + 1].Z = BottomOfSky + 1;

                CapB[i + 1].Tu = 0.0f; // SkyWidth * (float)i;
                CapB[i + 1].Tv = 0.99999f;


            }

            CapA[0].X = 0;
            CapA[0].Y = 0;
            CapA[0].Z = TopOfSky + 150.0f;
            CapA[0].Tu = 0.0f;
            CapA[0].Tv = 0.0f;

            CapB[0].X = 0;
            CapB[0].Y = 0;
            CapB[0].Z = BottomOfSky - 150.0f;
            CapB[0].Tu = 0.0f;
            CapB[0].Tv = 0.99999f;
            


            GraphicsStream Writer = SkyBuffer.Lock(0, 0, 0);
            Writer.Write(Verts);
            SkyBuffer.Unlock();

            for (int i = 0; i < 2; i++) {
                Writer = SkyBufferCap[i].Lock(0, 0, 0);
                Writer.Write(i == 0 ? CapA : CapB);
                SkyBufferCap[i].Unlock();
            }
            
        }


        public void DrawSky() {
            RenderDevice.RenderState.ZBufferWriteEnable = false;
            RenderDevice.VertexFormat = CustomVertex.PositionTextured.Format;
            
            RenderDevice.SetTexture(0, SkyTexture);

            for (int i = 0; i < 2; i++) {
                RenderDevice.SetStreamSource(0, SkyBufferCap[i], 0);
                RenderDevice.DrawPrimitives(PrimitiveType.TriangleFan, 0, NumSkySlices);
            }

            RenderDevice.SetStreamSource(0, SkyBuffer, 0);
            RenderDevice.DrawPrimitives(PrimitiveType.TriangleStrip, 0, NumSkySlices * 2);


            


            RenderDevice.RenderState.ZBufferWriteEnable = true;
        }
    }
}
