using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.Collections.Generic;
using System.IO;

using DOOM.WAD;
using DOOM.Levels;

namespace DOOM {
    public partial class DOOM {

        public Sprite SpriteRenderer = null;


        public List<SpriteTexture> SpriteTextures;
        public List<WorkingThing> WorkingThings;

        public class SpriteTexture {
            public readonly Texture Image;
            public readonly Point Origin;
            public readonly int Width;
            public readonly int Height;
            public SpriteTexture(Texture Image, Point Origin, int Width, int Height) {
                this.Image = Image;
                this.Origin = Origin;
            }
        }

        public class WorkingThing {
            public Vector3 Location = new Vector3();
            public int TextureIndex = -1;
            public bool Visible = false;
            public int TextureWidth;
            public int TextureHeight;

        }

        public void InitialiseThingTextures() {
            
            Hashtable SpriteNames = new Hashtable();
            SpriteTextures = new List<SpriteTexture>();
            WorkingThings = new List<WorkingThing>();

            
            foreach (Wad.Thing T in this.CurrentLevel.Things) {

                WorkingThing WT = new WorkingThing();

                WT.Visible = false;

                if (T.SpriteName != null) {
                    WT.Visible = true;
                    
                    object FindTexture = SpriteNames[T.SpriteName];
                    if (FindTexture == null) {
                        foreach (Wad.Lump L in CurrentLevel.WadFile.Lumps) {
                            if (L.Type == Wad.LumpType.Graphic && L.Name.StartsWith(T.SpriteName)) {

                                SpriteNames[T.SpriteName] = SpriteTextures.Count;
                                WT.TextureIndex = SpriteTextures.Count;


                                MemoryStream M = new MemoryStream();

                                Wad.Graphic WG = (Wad.Graphic)L.DecodedData;

                                WG.Image.Save(M, ImageFormat.Png);
                                M.Seek(0, SeekOrigin.Begin);

                                SpriteTextures.Add(new SpriteTexture(TextureLoader.FromStream(RenderDevice, M), WG.Origin, WG.Image.Width, WG.Image.Height));
                                break;
                            }
                        }
                    } else {
                        WT.TextureIndex = (int)FindTexture;
                    }
                }

                WT.Location = new Vector3(T.Position.X, T.Position.Y, 0.0f);
                

                WorkingThings.Add(WT);

            } 
        }

        public void DrawThings() {

            SpriteRenderer.SetWorldViewRH(RenderDevice.Transform.World, RenderDevice.Transform.View);
            SpriteRenderer.Begin(SpriteFlags.SortDepthBackToFront | SpriteFlags.ObjectSpace | SpriteFlags.AlphaBlend | SpriteFlags.Billboard | SpriteFlags.DoNotModifyRenderState);
            foreach (WorkingThing WT in WorkingThings) {
                if (WT.Visible) {
                    SpriteRenderer.Draw(SpriteTextures[WT.TextureIndex].Image, new Vector3(0.0f, 0.0f, -1.0f), WT.Location, 0xFFFFFF);
                }
            }
            SpriteRenderer.End();
        }
    }
}
