using System;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using System.Drawing;


using DOOM.WAD;
using System.Collections.Generic;

namespace DOOM {
    public partial class DOOM {

        Timer AnimateTextures = null;

        private void CleanUpUpdateEnvironment() {
            if (AnimateTextures != null) AnimateTextures.Stop();
            if (BlinkRandomlyTimers != null) foreach (Timer T in BlinkRandomlyTimers) T.Stop();
            if (OscillateTimer != null) OscillateTimer.Stop();
            if (BlinkHalfTimer != null) BlinkHalfTimer.Stop();
            if (BlinkFullTimer != null) BlinkFullTimer.Stop();
        }

        private class SectorLightingEffect {
            public List<SectorComponent> SectorComponents = null;
            public int MaxLightLevel = 0;
            public int MinLightLevel = 0;
            public int CurrentLightLevel = 0xFF;
            public bool IsOn = false;
            public SectorLightingEffect(List<SectorComponent> SectorComponents, int MaxLightLevel, int MinLightLevel) {
                this.SectorComponents = SectorComponents;
                this.MaxLightLevel = MaxLightLevel;
                this.MinLightLevel = MinLightLevel;
                this.CurrentLightLevel = MaxLightLevel;
            }
        }

        private List<Timer> BlinkRandomlyTimers = null;
        //private List<SectorLightingEffect> BlinkRandomly = null;

        private Timer OscillateTimer = null;
        private List<SectorLightingEffect> Oscillate = null;

        private Timer BlinkHalfTimer = null;
        private List<SectorLightingEffect> BlinkHalf = null;
        
        private Timer BlinkFullTimer = null;
        private List<SectorLightingEffect> BlinkFull = null;

        private Random RandomFlickering = new Random();

        public void StartUpdateEnvironment() {
            AnimateTextures = new Timer();
            AnimateTextures.Interval = 300;
            AnimateTextures.Tick += new EventHandler(AnimateTextures_Tick);
            AnimateTextures.Enabled = true;

            // Randomly blinking off lights:
            BlinkRandomlyTimers = new List<Timer>();
            foreach (Wad.Sector S in CurrentLevel.Sectors) {
                if (S.Type == Wad.Sector.SectorType.LightRandomOff) {
                    int LowestLevel = 255;
                    foreach (Wad.Sector A in S.AdjacentSectors) {
                        LowestLevel = Math.Min(LowestLevel, A.LightLevel);                        
                    }
                    if (S.AdjacentSectors.Count == 0 || LowestLevel > S.LightLevel) {
                        LowestLevel = 0;
                    }
                    Timer BlinkRandomlyTimer = new Timer();
                    BlinkRandomlyTimer.Interval = 100;
                    BlinkRandomlyTimer.Tick += new EventHandler(BlinkRandomlyTimer_Tick);
                    BlinkRandomlyTimer.Enabled = true;
                    BlinkRandomlyTimer.Tag = new SectorLightingEffect(SectorToVerticesLinkup[S], S.LightLevel, LowestLevel); 
                    BlinkRandomlyTimers.Add(BlinkRandomlyTimer);
                }
            }
  

            // Smoothly oscillating lights:
            Oscillate = new List<SectorLightingEffect>();
            foreach (Wad.Sector S in CurrentLevel.Sectors) {
                if (S.Type == Wad.Sector.SectorType.LightOscillates) {
                    int LowestLevel = 255;
                    foreach (Wad.Sector A in S.AdjacentSectors) LowestLevel = Math.Min(LowestLevel, A.LightLevel);
                    if (S.AdjacentSectors.Count == 0 || LowestLevel > S.LightLevel) continue;
                    List<SectorComponent> Try;
                    if (SectorToVerticesLinkup.TryGetValue(S, out Try)) {
                        Oscillate.Add(new SectorLightingEffect(Try, S.LightLevel, LowestLevel));
                    }
                }
            }
            if (Oscillate.Count != 0) {
                OscillateTimer = new Timer();
                OscillateTimer.Interval = 40;
                OscillateTimer.Tick += new EventHandler(OscillateTimer_Tick);
                OscillateTimer.Enabled = true;
            }

            // Blinking on every half-second:
            BlinkHalf = new List<SectorLightingEffect>();
            foreach (Wad.Sector S in CurrentLevel.Sectors) {
                if (S.Type == Wad.Sector.SectorType.LightBlinkHalfSecond || S.Type == Wad.Sector.SectorType.LightBlinkHalfSecondSynchronized) {
                    int LowestLevel = 255;
                    foreach (Wad.Sector A in S.AdjacentSectors) LowestLevel = Math.Min(LowestLevel, A.LightLevel);
                    if (S.AdjacentSectors.Count == 0 || LowestLevel > S.LightLevel) LowestLevel = 0;
                    BlinkHalf.Add(new SectorLightingEffect(SectorToVerticesLinkup[S], S.LightLevel, LowestLevel));
                }
            }
            if (BlinkHalf.Count != 0) {
                BlinkHalfTimer = new Timer();
                BlinkHalfTimer.Interval = 500;
                BlinkHalfTimer.Tick += new EventHandler(BlinkHalfTimer_Tick);
                BlinkHalfTimer.Enabled = true;
            }

            // Blinking on every second:
            BlinkFull = new List<SectorLightingEffect>();
            foreach (Wad.Sector S in CurrentLevel.Sectors) {
                if (S.Type == Wad.Sector.SectorType.LightBlinkWholeSecond || S.Type == Wad.Sector.SectorType.LightBlinkWholeSecondSynchronized) {
                    
                    int LowestLevel = 255;
                    foreach (Wad.Sector A in S.AdjacentSectors) LowestLevel = Math.Min(LowestLevel, A.LightLevel);
                    if (S.AdjacentSectors.Count == 0 || LowestLevel > S.LightLevel) LowestLevel = 0;
                    BlinkFull.Add(new SectorLightingEffect(SectorToVerticesLinkup[S], S.LightLevel, LowestLevel));
                }
            }
            if (BlinkFull.Count != 0) {
                BlinkFullTimer = new Timer();
                BlinkFullTimer.Interval = 1000;
                BlinkFullTimer.Tick += new EventHandler(BlinkFullTimer_Tick);
                BlinkFullTimer.Enabled = true;
            }

        }

        private DateTime SinceLastScrolled = DateTime.Now;

        private void Blink(ref Timer sender, int Duration, List<SectorLightingEffect> Sector) {
            bool IsOn = false;
            foreach (SectorLightingEffect FX in Sector) {
                FX.IsOn = !FX.IsOn;
                IsOn = FX.IsOn;
                int LightColour = FX.IsOn ? FX.MaxLightLevel | (FX.MaxLightLevel << 8) | (FX.MaxLightLevel << 16) : FX.MinLightLevel | (FX.MinLightLevel << 8) | (FX.MinLightLevel << 16);
                foreach (SectorComponent S in FX.SectorComponents) {
                    for (int i = 0; i < S.Vertices.Length; i++) {
                        S.Vertices[i].Color = LightColour;
                    }
                    GraphicsStream Adjust = S.Buffer.Lock(S.Offset, S.Length, LockFlags.None);
                    Adjust.Write(S.Vertices);
                    S.Buffer.Unlock();
                }
            }
            sender.Interval = IsOn ? Duration / 5 : Duration - Duration / 5;
        }

        void BlinkFullTimer_Tick(object sender, EventArgs e) {
            Blink(ref BlinkFullTimer, 1000, BlinkFull);
        }

        void BlinkHalfTimer_Tick(object sender, EventArgs e) {
            Blink(ref BlinkHalfTimer, 500, BlinkHalf);
        }

        void OscillateTimer_Tick(object sender, EventArgs e) {
            foreach (SectorLightingEffect FX in Oscillate) {
                
                FX.CurrentLightLevel += FX.IsOn ? +8 : -8;
                if (FX.CurrentLightLevel > FX.MaxLightLevel) {
                    FX.CurrentLightLevel = FX.MaxLightLevel;
                    FX.IsOn = false;
                } else if (FX.CurrentLightLevel < FX.MinLightLevel) {
                    FX.CurrentLightLevel = FX.MinLightLevel;
                    FX.IsOn = true;
                }

                int LightColour = FX.CurrentLightLevel | (FX.CurrentLightLevel << 8) | (FX.CurrentLightLevel << 16);
                foreach (SectorComponent S in FX.SectorComponents) {
                    for (int i = 0; i < S.Vertices.Length; i++) {
                        S.Vertices[i].Color = LightColour;
                    }
                    GraphicsStream Adjust = S.Buffer.Lock(S.Offset, S.Length, LockFlags.None);
                    Adjust.Write(S.Vertices);
                    S.Buffer.Unlock();
                }           
            }
        }

        void BlinkRandomlyTimer_Tick(object sender, EventArgs e) {
            Timer T = (Timer)sender;
            SectorLightingEffect FX = (SectorLightingEffect)T.Tag;
            FX.IsOn = !FX.IsOn;
            int LightColour = FX.IsOn ? FX.MaxLightLevel + (FX.MaxLightLevel << 8) + (FX.MaxLightLevel << 16) : FX.MinLightLevel + (FX.MinLightLevel << 8) + (FX.MinLightLevel << 16);
            foreach (SectorComponent S in FX.SectorComponents) {
                for (int i = 0; i < S.Vertices.Length; i++) {
                    S.Vertices[i].Color = LightColour;
                }
                GraphicsStream Adjust = S.Buffer.Lock(S.Offset, S.Length, LockFlags.None);
                Adjust.Write(S.Vertices);
                S.Buffer.Unlock();
            }
            T.Interval = RandomFlickering.Next(FX.IsOn ? 50 : 10, FX.IsOn ? 2000 : 100);
        }

        void AnimateTextures_Tick(object sender, EventArgs e) {
            foreach (TextureAnimation T in this.SolidWalls) {
                T.TextureIndex = (T.TextureIndex + 1) % T.TextureList.Count;
            }
            foreach (TextureAnimation T in this.TransparentWalls) {
                T.TextureIndex = (T.TextureIndex + 1) % T.TextureList.Count;
            }
            foreach (TextureAnimation T in this.Floors) {
                T.TextureIndex = (T.TextureIndex + 1) % T.TextureList.Count;
            }
        }
        public void UpdateEnvironment() {
            DateTime NowScrolling = DateTime.Now;
            TimeSpan TimeChange = NowScrolling - SinceLastScrolled;
            SinceLastScrolled = NowScrolling;
            float WallScrollDelta = (float)TimeChange.TotalMilliseconds / 1280.0f;

            foreach (TextureAnimation T in SolidWalls) {
                foreach (int i in T.ScrollingWallOffsets) {
                    GraphicsStream GS = T.Geometry.Lock(i * CustomVertex.PositionColoredTextured.StrideSize, 6 * CustomVertex.PositionColoredTextured.StrideSize, LockFlags.None);
                    CustomVertex.PositionColoredTextured[] Vertices = (CustomVertex.PositionColoredTextured[])GS.Read(typeof(CustomVertex.PositionColoredTextured), 6);
                    for (int j = 0; j < 6; j++) {
                        Vertices[j].Tu -= WallScrollDelta;
                    }
                    GS.Seek(0, System.IO.SeekOrigin.Begin);
                    GS.Write(Vertices);
                    T.Geometry.Unlock();
                }
            }
            
        }



    }
}
