// Segment decoding
using System;
using System.Collections.Generic;
using System.Drawing;

namespace DOOM.WAD {
    public partial class Wad {
        
        public class AnimatedTexture {
            public List<WallTexture> AllTextures;
            public List<string> AllTextureNames;
            public readonly string BaseTexture;
            
            public AnimatedTexture(Wad.WallTexture Frame1) {
                this.BaseTexture = Frame1.Name;
                AllTextures = new List<WallTexture>();
                AllTextureNames = new List<string>();
                AllTextures.Add(Frame1);
                AllTextureNames.Add(Frame1.Name);
            }
        }


        public class AnimatedFlat {
            public List<Wad.Lump> AllTextures;
            public List<string> AllTextureNames;
            public readonly string BaseTexture;
            public AnimatedFlat(Wad.Lump Frame1) {
                this.BaseTexture = Frame1.Name;
                AllTextures = new List<Lump>();
                AllTextureNames = new List<string>();
                AllTextures.Add(Frame1);
                AllTextureNames.Add(Frame1.Name);

            }
        }

        public AnimatedTexture[] AnimatedTextures = null;
        public AnimatedFlat[] AnimatedFlats = null;


        private void WorkOutWhichAreAnimatedTextures() {

            List<AnimatedTexture> FindAnimatedTextures = new List<AnimatedTexture>();
            List<AnimatedFlat> FindAnimatedFlats = new List<AnimatedFlat>();

            string[] AnimatedNames = {
                "BLODGR1",  "BLODGR4",
                "BLODRIP1", "BLODRIP4",
                "FIREBLU1", "FIREBLU2",
                "FIRELAV3", "FIRELAVA",
                "FIREMAG1", "FIREMAG3",
                "FIREWALA", "FIREWALL",
                "GSTFONT1", "GSTFONT3",
                "ROCKRED1", "ROCKRED3",
                "SLADRIP1", "SLADRIP3",
                "BFALL1",   "BFALL4",
                "SFALL1",   "SFALL4",
                "WFALL1",   "WFALL4",
                "DBRAIN1",  "DBRAIN4",
                "NUKAGE1",  "NUKAGE3",
                "FWATER1",  "FWATER4",
                "SWATER1",  "SWATER4",
                "LAVA1",    "LAVA4",
                "BLOOD1",   "BLOOD3",
                "RROCK05",  "RROCK08",
                "SLIME01",  "SLIME04",
                "SLIME05",  "SLIME08",
                "SLIME09",  "SLIME12"
            };

            bool IsAnimated;
            
            
            IsAnimated = false;
            AnimatedTexture CurrentAdder = null;
            foreach (string o in this.TextureNames) {
                WallTexture T = (WallTexture)this.Textures[o];
                bool FoundAndAdded = false;
                for (int i = 0; i < AnimatedNames.Length; ++i) {
                    if (AnimatedNames[i] == T.Name) {
                        switch (i & 1) {
                            case 0:
                                CurrentAdder = new AnimatedTexture(T);
                                IsAnimated = true;
                                FoundAndAdded = true;
                                break;
                            case 1:
                                CurrentAdder.AllTextures.Add(T);
                                CurrentAdder.AllTextureNames.Add(T.Name);
                                FindAnimatedTextures.Add(CurrentAdder);
                                CurrentAdder = null;
                                IsAnimated = false;
                                FoundAndAdded = true;
                                break;
                        }
                    }
                }
                if (!FoundAndAdded) {
                     if (IsAnimated && CurrentAdder != null) {
                        CurrentAdder.AllTextures.Add(T);
                        CurrentAdder.AllTextureNames.Add(T.Name);
                    }
                }
            }

            IsAnimated = false;
            AnimatedFlat CurrentFlatAdder = null;
            foreach (Lump L in this.Lumps) {
                if (L.Type != LumpType.Flat) continue;
                bool FoundAndAdded = false;
                for (int i = 0; i < AnimatedNames.Length; ++i) {
                    if (AnimatedNames[i] == L.Name) {
                        switch (i & 1) {
                            case 0:
                                CurrentFlatAdder = new AnimatedFlat(L);
                                IsAnimated = true;
                                FoundAndAdded = true;
                                break;
                            case 1:
                                CurrentFlatAdder.AllTextures.Add(L);
                                CurrentFlatAdder.AllTextureNames.Add(L.Name);
                                FindAnimatedFlats.Add(CurrentFlatAdder);
                                CurrentFlatAdder = null;
                                IsAnimated = false;
                                FoundAndAdded = true;
                                break;
                        }
                    }
                }
                if (!FoundAndAdded) {
                    if (IsAnimated && CurrentFlatAdder != null) {
                        CurrentFlatAdder.AllTextures.Add(L);
                        CurrentFlatAdder.AllTextureNames.Add(L.Name);
                    }
                }
            }
            this.AnimatedTextures = FindAnimatedTextures.ToArray();
            this.AnimatedFlats = FindAnimatedFlats.ToArray();
            Console.WriteLine("Animated textures: {0} (walls) {1} (flats)", AnimatedTextures.Length, AnimatedFlats.Length);
        }
    }
}