// Graphic decoding
using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace DOOM.WAD {
    public partial class Wad {
        public static Graphic DecodeGraphic(Lump DecodeGraphic) {

            int Width = DecodeGraphic.Data[0] + (DecodeGraphic.Data[1] << 8);
            int Height = DecodeGraphic.Data[2] + (DecodeGraphic.Data[3] << 8);
            Point Origin = new Point(
                DecodeGraphic.Data[4] + (DecodeGraphic.Data[5] << 8),
                DecodeGraphic.Data[6] + (DecodeGraphic.Data[7] << 8));

            Bitmap Image = new Bitmap(Width, Height, PixelFormat.Format32bppArgb);


            bool IsPrerelease = false;
            bool DecodeFailed = true;

            while (DecodeFailed) {
                BitmapData Locked = Image.LockBits(new Rectangle(0, 0, Width, Height), ImageLockMode.WriteOnly, PixelFormat.Format32bppArgb);
                DecodeFailed = false;
                unsafe {

                    int* q = (int*)Locked.Scan0.ToPointer();
                    int* p = q;

                    int* ImageStart = p;
                    int* ImageEnd = p + ((Locked.Stride * Locked.Height) >> 2);

                    int DrawingX = 0;

                    for (int x = 0; x < Width; ++x) {
                        int ColumnStart;
                        if (IsPrerelease) {
                            ColumnStart =
                                (DecodeGraphic.Data[8 + x * 2 + 0] << 00) +
                                (DecodeGraphic.Data[8 + x * 2 + 1] << 08);
                        } else {
                            ColumnStart =
                                (DecodeGraphic.Data[8 + x * 4 + 0] << 00) +
                                (DecodeGraphic.Data[8 + x * 4 + 1] << 08) +
                                (DecodeGraphic.Data[8 + x * 4 + 2] << 16) +
                                (DecodeGraphic.Data[8 + x * 4 + 3] << 24);
                        }

                        for (; ; ) {
                            if (ColumnStart < 0 || ColumnStart >= DecodeGraphic.Data.Length) {
                                DecodeFailed = true;
                                break;
                            }
                            byte StartRow = DecodeGraphic.Data[ColumnStart++];

                            if (StartRow == 0xFF) {
                                ++DrawingX;
                                p = q + DrawingX;
                                break;
                            }

                            p = q + StartRow * Width + DrawingX;

                            if (ColumnStart > DecodeGraphic.Data.Length || ColumnStart < 0) break;
                            int NumPixels = (int)DecodeGraphic.Data[ColumnStart++];

                            if (!IsPrerelease) ++ColumnStart; // Skip

                            for (int i = 0; i < NumPixels; ++i) {
                                if (ColumnStart > DecodeGraphic.Data.Length || ColumnStart < 0) break;
                                if (p >= ImageStart && p < ImageEnd) {
                                    int Colour = DecodeGraphic.Parent.Palettes[0].Colours[DecodeGraphic.Data[ColumnStart++]].ToArgb();
                                    *p = Colour;
                                    p += Width;
                                } else {
                                    DecodeFailed = true;
                                    break;
                                }
                            }
                            if (!IsPrerelease) ++ColumnStart; // Skip
                            if (DecodeFailed) break;
                        }

                    }
                }
                Image.UnlockBits(Locked);
                if (DecodeFailed && IsPrerelease) {
                    break;
                }
                IsPrerelease = true;
            }

            return new Graphic(Image, Origin);

        }

        public class Graphic {
            public readonly Bitmap Image;
            public readonly Point Origin;
            public Graphic(Bitmap Image, Point Origin) {
                this.Image = Image;
                this.Origin = Origin;
            }
        }
    }
}