// LineDef decoding
using System;

namespace DOOM.WAD {
    public partial class Wad {
        public static Line[] DecodeLine(Lump LineDef) {
            Line[] Return = new Line[LineDef.Size / 14];
            for (int i = 0; i < LineDef.Size; i += 14) {
                Return[i / 14] = new Line(
                    i / 14,
                    LineDef.Data[i + 0] + (LineDef.Data[i + 1] << 8),
                    LineDef.Data[i + 2] + (LineDef.Data[i + 3] << 8),
                    (Line.LineFlags)LineDef.Data[i + 4] + (LineDef.Data[i + 5] << 8),
                    LineDef.Data[i + 6] + (LineDef.Data[i + 7] << 8),
                    LineDef.Data[i + 8] + (LineDef.Data[i + 9] << 8),
                    FixSignedShort(LineDef.Data[i + 10] + (LineDef.Data[i + 11] << 8)),
                    FixSignedShort(LineDef.Data[i + 12] + (LineDef.Data[i + 13] << 8))
                );
            }
            return Return;
        }

        public class Line {

            /// <summary>
            /// Flags associated with a particular linedef
            /// </summary>
            [Flags]
            public enum LineFlags {
                /// <summary>
                /// Players and monsters cannot cross this line.
                /// Note that if there is no sector on the other side, they can't go through the line anyway, regardless of the flags.
                /// </summary>
                Impassable = 0x0001,
                /// <summary>
                /// Monsters cannot cross this line.
                /// </summary>
                BlockMonsters = 0x0002,
                TwoSided = 0x0004,
                UpperUnpegged = 0x0008,
                LowerUnpegged = 0x0010,
                /// <summary>
                /// Drawn as red on the automap.
                /// </summary>
                Secret = 0x0020,
                /// <summary>
                /// Sound travels from sector to sector, but stops after passing 2 linedefs marked as blocking sound.
                /// </summary>
                BlockSound = 0x0040,
                /// <summary>
                /// This line will never appear on the automap.
                /// </summary>
                NotOnMap = 0x0080,
                /// <summary>
                /// This line is automatically drawn on the automap at the start of the level.
                /// </summary>
                AlreadyOnMap = 0x0100
                
            };

            public enum LineType {
                /// <summary>
                /// No special action
                /// </summary>
                None = 0x00,

                /// <summary>
                /// Texture scrolls along the wall
                /// </summary>
                ScrollingWall = 48,




            }

            public readonly int ID;
            public readonly int StartVertex;
            public readonly int EndVertex;
            public readonly LineFlags Flags;
            public readonly LineType Type;
            public readonly int Tag;
            public readonly int LeftSide;
            public readonly int RightSide;
            public int LeftSector = -1;
            public int RightSector = -1;

            public Line(int ID, int StartVertex, int EndVertex, LineFlags Flags, int Type, int Tag, int RightSide, int LeftSide) {
                this.ID = ID;
                this.StartVertex = StartVertex;
                this.EndVertex = EndVertex;
                this.Flags = Flags;
                this.Type = (LineType)Type;
                this.Tag = Tag;
                this.RightSide = RightSide;
                this.LeftSide = LeftSide;
            }
        }
    }
}