// Nodes decoding
using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections.Generic;

namespace DOOM.WAD {
    public partial class Wad {

        public static BspNode[] DecodeNodes(Lump NodesFile) {
            BspNode[] Nodes = new BspNode[NodesFile.Size / 28];

            for (int i = 0; i < NodesFile.Size; i += 28) {
                Point Start = new Point(
                    FixSignedShort(NodesFile.Data[i + 0] + (NodesFile.Data[i + 1] << 8)),
                    FixSignedShort(NodesFile.Data[i + 2] + (NodesFile.Data[i + 3] << 8))
                );
                Point End = new Point(
                    Start.X + FixSignedShort(NodesFile.Data[i + 4] + (NodesFile.Data[i + 5] << 8)),
                    Start.Y + FixSignedShort(NodesFile.Data[i + 6] + (NodesFile.Data[i + 7] << 8))
                );

                int RightChild = NodesFile.Data[i + 24] + (NodesFile.Data[i + 25] << 8);
                int LeftChild = NodesFile.Data[i + 26] + (NodesFile.Data[i + 27] << 8);

                Nodes[i / 28] = new BspNode(i / 28, Start, End, RightChild, LeftChild);
                
            }
            return Nodes;
        }
        public class BspNode {

            /// <summary>
            /// Identifier
            /// </summary>
            public readonly int ID;


            /// <summary>
            /// Start of the partition line
            /// </summary>
            public readonly Point Start;
            /// <summary>
            /// End of the partition line
            /// </summary>
            public readonly Point End;

            /// <summary>
            /// Rightmost segs must be within this box.
            /// </summary>
            public readonly Rectangle RightBoundingBox;
            /// <summary>
            /// Leftmost segs must be within this box.
            /// </summary>
            public readonly Rectangle LeftBoundingBox;

            public readonly int RightNode = -1;
            public readonly int LeftNode = -1;

            public readonly int RightSubSector = -1;
            public readonly int LeftSubSector = -1;
            
            public BspNode(int ID, Point Start, Point End, int Right, int Left) {
                this.ID = ID;
                this.Start = Start;
                this.End = End;
                if ((Right & 0x8000) == 0) {
                    this.RightNode = Right;
                } else {
                    this.RightSubSector = Right & 0x7FFF;
                }
                if ((Left & 0x8000) == 0) {
                    this.LeftNode = Left;
                } else {
                    this.LeftSubSector = Left & 0x7FFF;
                }
            }
        }
    }
}