// Segment decoding
using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace DOOM.WAD {
    public partial class Wad {

        public static Segment[] DecodeSegments(Lump SegFile) {
            Segment[] Return = new Segment[SegFile.Size / 12];
            for (int i = 0; i < SegFile.Size; i += 12) {
                Return[i / 12] = new Segment(
                    SegFile.Data[i + 0] + (SegFile.Data[i + 1] << 8),
                    SegFile.Data[i + 2] + (SegFile.Data[i + 3] << 8),
                    FixSignedShort(SegFile.Data[i + 4] + (SegFile.Data[i + 5] << 8)),
                    SegFile.Data[i + 6] + (SegFile.Data[i + 7] << 8),
                    (SegFile.Data[i + 8] + (SegFile.Data[i + 9] << 8)) != 0,
                    SegFile.Data[i + 10] + (SegFile.Data[i + 11] << 8)
                );
            }
            return Return;
        }
        public class Segment {
            /// <summary>
            /// The index of the vertex at the start of the segment
            /// </summary>
            public readonly int StartVertex;
            /// <summary>
            /// The index of the vertex at the end of the segment
            /// </summary>
            public readonly int EndVertex;

            /// <summary>
            /// Angle the segment points
            /// </summary>
            public readonly int Angle;

            /// <summary>
            /// The index of the linedef that the segment goes along
            /// </summary>
            public readonly int Line;

            /// <summary>
            /// Records whether this segment goes in the same direction as the line or not
            /// </summary>
            public readonly bool OppositeDirectionToLine;

            /// <summary>
            /// How far along the linedef the start of the seg is
            /// </summary>
            public readonly int Offset;

            public Segment(int StartVertex, int EndVertex, int Angle, int Line, bool OppositeDirectionToLine, int Offset) {
                this.StartVertex = StartVertex;
                this.EndVertex = EndVertex;
                this.Angle = Angle;
                this.Line = Line;
                this.OppositeDirectionToLine = OppositeDirectionToLine;
                this.Offset = Offset;
            }
            
        }
    }
}