// LineDef decoding
using System;
using System.Drawing;

namespace DOOM.WAD {
    public partial class Wad {
        public static Side[] DecodeSide(Lump SideDef) {
            Side[] Return = new Side[SideDef.Size / 30];
            for (int i = 0; i < SideDef.Size; i += 30) {
                int X = FixSignedShort(SideDef.Data[i + 0] + (SideDef.Data[i + 1] << 8));
                int Y = FixSignedShort(SideDef.Data[i + 2] + (SideDef.Data[i + 3] << 8));

                WallTexture[] FindTextures = new WallTexture[] { null, null, null };                

                
                for (int j = 0; j < 3; ++j) {
                    string TextureName = "";
                    for (int k = 0; k < 8; k++) {
                        TextureName += (char)SideDef.Data[k + j * 8 + 4 + i];
                    }
                    TextureName = Wad.FixString(TextureName);

                    if (TextureName != "-") {
                        object FindTexture = SideDef.Parent.Textures[TextureName];
                        if (FindTexture != null) {
                            FindTextures[j] = (WallTexture)FindTexture;
                        } else {
                            Console.WriteLine("Texture " + TextureName + " not found.");
                        }
                    }
                }

                int Sector = SideDef.Data[i + 28] + (SideDef.Data[i + 29] << 8);

                Return[i / 30] = new Side(i / 30, X, Y, FindTextures[0], FindTextures[1], FindTextures[2], Sector);

            }
            return Return;
        }


        public class Side {

            /// <summary>
            /// Identifier
            /// </summary>
            public readonly int ID;

            /// <summary>
            /// Texture X-offset
            /// </summary>
            public readonly int X;
            /// <summary>
            /// Texture Y-offset
            /// </summary>
            public readonly int Y;

            /// <summary>
            /// Upper texture lump index
            /// </summary>
            public readonly WallTexture UpperTexture;
            /// <summary>
            /// Lower texture lump index
            /// </summary>
            public readonly WallTexture LowerTexture;
            /// <summary>
            /// Middle texture lump index
            /// </summary>
            public readonly WallTexture MiddleTexture;

            /// <summary>
            /// Sector that this side surrounds or faces
            /// </summary>
            public readonly int Sector;


            public Side(int ID, int X, int Y, WallTexture UpperTexture, WallTexture LowerTexture, WallTexture MiddleTexture, int Sector) {
                this.ID = ID;
                this.X = X;
                this.Y = Y;
                this.UpperTexture = UpperTexture;
                this.LowerTexture = LowerTexture;
                this.MiddleTexture = MiddleTexture;
                this.Sector = Sector;
            }
        }
    }
}