// Texture decoding
using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
namespace DOOM.WAD {
    public partial class Wad {

        public Hashtable Textures = null;

        private int IndividualTextureId = 0;
        public void DecodeTextures(Lump TextureLump) {
            int TextureCount =
                (TextureLump.Data[0] << 00) +
                (TextureLump.Data[1] << 08) +
                (TextureLump.Data[2] << 16) +
                (TextureLump.Data[3] << 24);
            
            
            for (int i = 0; i < TextureCount; ++i) {

                int DataPointer =
                    (TextureLump.Data[i * 4 + 4] << 00) +
                    (TextureLump.Data[i * 4 + 5] << 08) +
                    (TextureLump.Data[i * 4 + 6] << 16) +
                    (TextureLump.Data[i * 4 + 7] << 24);

                string Name = "";
                for (int j = 0; j < 8; ++j) {
                    Name += (char)TextureLump.Data[DataPointer++];                   
                }
                Name = FixString(Name);
                
                DataPointer += 4;
                
                int Width = TextureLump.Data[DataPointer + 0] + (TextureLump.Data[DataPointer + 1] << 8); DataPointer += 2;
                int Height = TextureLump.Data[DataPointer + 0] + (TextureLump.Data[DataPointer + 1] << 8); DataPointer += 2;

                DataPointer += 4;

                int PatchDescriptorCount = TextureLump.Data[DataPointer + 0] + (TextureLump.Data[DataPointer + 1] << 8); DataPointer += 2;

                PatchDescriptor[] P = new PatchDescriptor[PatchDescriptorCount];

                for (int j = 0; j < PatchDescriptorCount; ++j) {
                    P[j] = new PatchDescriptor(
                        new Point(FixSignedShort(TextureLump.Data[DataPointer + 0] + (TextureLump.Data[DataPointer + 1] << 8)), FixSignedShort(TextureLump.Data[DataPointer + 2] + (TextureLump.Data[DataPointer + 3] << 8))),
                        TextureLump.Data[DataPointer + 4] + (TextureLump.Data[DataPointer + 5] << 8)
                    );
                    DataPointer += 10;
                }
                Textures[Name] = new WallTexture(this, IndividualTextureId++, Name, Width, Height, P);
                this.TextureNames.Add(Name);
            }
        }
        private List<string> TextureNames = new List<string>();

        public class WallTexture {
            
            /// <summary>
            /// Identifier
            /// </summary>
            public readonly int ID;

            /// <summary>
            /// Name of the texture
            /// </summary>
            public readonly string Name;

            /// <summary>
            /// Width of the texture
            /// </summary>
            public readonly int Width;
            /// <summary>
            /// Height of the texture
            /// </summary>
            public readonly int Height;

            /// <summary>
            /// The list of patches used to build up the texture.
            /// </summary>
            private readonly PatchDescriptor[] Patches;

            public Bitmap Texture {
                get {
                    if (_Texture != null) {
                        return _Texture;
                    } else {
                        // Decode!
                        Bitmap B = new Bitmap(Width, Height);
                        Graphics G = Graphics.FromImage(B);

                        foreach (PatchDescriptor P in this.Patches) {
                            Graphic ToDraw = (Graphic)Parent.Lumps[Parent.PatchIndices[P.PatchIndex]].DecodedData;
                            G.DrawImage(ToDraw.Image, P.Position);
                        }
                        _Texture = B;
                        return _Texture;
                    }

                }
            }
            private Bitmap _Texture = null;

            private readonly Wad Parent;

            public WallTexture(Wad Parent, int ID, string Name, int Width, int Height, PatchDescriptor[] Patches) {
                this.Parent = Parent;
                this.ID = ID;
                this.Name = Name;
                this.Width = Width;
                this.Height = Height;
                this.Patches = Patches;
            }
        }

        public class PatchDescriptor {
            public readonly Point Position;
            public readonly int PatchIndex;
            public PatchDescriptor(Point Position, int PatchIndex) {
                this.Position = Position;
                this.PatchIndex = PatchIndex;
            }

        }
    }
}
