// Thing decoding
using System;
using System.Drawing;

namespace DOOM.WAD {
    public partial class Wad {
        public static Thing[] DecodeThings(Lump ThingsFile) {
            Thing[] Return = new Thing[ThingsFile.Size / 10];
            for (int i = 0; i < ThingsFile.Size; i += 10) {
                Return[i / 10] = new Thing(
                    i / 10,
                    new Point(
                        FixSignedShort(ThingsFile.Data[i + 0] + (ThingsFile.Data[i + 1] << 8)),
                        FixSignedShort(ThingsFile.Data[i + 2] + (ThingsFile.Data[i + 3] << 8))
                    ),
                    FixSignedShort(ThingsFile.Data[i + 4] + (ThingsFile.Data[i + 5] << 8)),
                    (Thing.ThingType)(ThingsFile.Data[i + 6] + (ThingsFile.Data[i + 7] << 8)),
                    (Thing.ThingOptions)(ThingsFile.Data[i + 8] + (ThingsFile.Data[i + 9] << 8))
                );
            }
            return Return;
        }
        public class Thing {
            public readonly int ID;
            public readonly Point Position;
            public readonly int Angle;
            public readonly ThingType Type;
            public readonly ThingOptions Options;
            
            public enum ThingType {
                Nothing = 0x0000,
                Nothing2 = 0xFFFF,
                
                Player1Start = 0x0001,
                Player2Start = 0x0002,
                Player3Start = 0x0003,
                Player4Start = 0x0004,
                DeathmatchStart = 0x000B,

                TeleportLanding = 0x000E,

                FormerHuman = 0x0BBC,
                WolfensteinSS = 0x0054,
                FormerHumanSergeant = 0x0009,
                HeavyWeaponDude = 0x0041,
                Imp = 0x0BB9,
                Demon = 0xBBA,
                Spectre = 0x003A,
                LostSoul = 0x0BBE,
                Cacodemon = 0x0BBD,
                HellKnight = 0x0045,
                BaronOfHell = 0x0BBB,
                Arachnotron = 0x0044,
                PainElemental= 0x0047,
                Revenant = 0x0042,
                Mancubus = 0x0043,
                ArchVile =0x0040,
                SpiderMastermind = 0x0007,
                CyberDemon = 0x0010,
                BossBrain = 0x0058,
                BossShooter = 0x0059,
                BossSpawnSpot = 0x0057,

                Chainsaw = 0x07D5,
                Shotgun = 0x07D1,
                SuperShotgun = 0x0052,
                Chaingun = 0x07D2,
                RocketLauncher = 0x07D3,
                PlasmaGun = 0x07D4,
                Bfg9000 = 0x07D6,

                AmmoClip = 0x07D7,
                ShotgunShells = 0x07D8,
                Rocket = 0x07DA,
                CellCharge = 0x7FF,
                AmmoCrate = 0x0800,
                ShellCrate = 0x0801,
                RocketCrate = 0x07FE,
                CellChargePack = 0x0011,
                Backpack = 0x0008,

                Stimpack = 0x07DB,
                Medikit = 0x07DC,
                HealthPotion = 0x07DE,
                SpiritArmour = 0x07DF,
                GreenArmour = 0x07E2,
                BlueArmour = 0x07E3,
                MegaSphere = 0x0053,
                SoulSphere = 0x07DD,
                Invulnerability = 0x07e6,
                Berserk = 0x07E7,
                Invisibility = 0x07E8,
                RadiationSuit = 0x7E9,
                ComputerMap = 0x7EA,
                LightAmplificationGoggles = 0x07FD,

                BlueKeycard = 0x0005,
                BlueSkullkey = 0x0028,
                RedKeycard = 0x000D,
                RedSkullkey = 0x0026,
                YellowKeycard = 0x0006,
                YellowSkullkey = 0x0027,

                Barrel = 0x07F3,

                CommanderKeen = 0x0048,

                TallTechnoPillar = 0x0030,
                TallGreenPillar = 0x001E,
                TallRedPillar = 0x0020,
                ShortGreenPillar = 0x001F,
                ShortGreenPillarWithHeart = 0x0024,
                ShortRedPillar = 0x0021,
                ShortRedPillarWithSkull = 0x0025,
                Stalagmite = 0x002F,
                BurntTree = 0x002B,
                LargeBrownTree = 0x0036,

                FloorLamp = 0x07EC,
                TallTechnoFloorLamp = 0x0055,
                ShortTechnoFloorLamp = 0x0056,
                Candle = 0x0022,
                Candelabra = 0x0023,
                TallBlueFirestick = 0x002C,
                TallGreenFirestick = 0x002D,
                TallRedFirestick = 0x002E,
                ShortBlueFirestick = 0x0037,
                ShortGreenFirestick  = 0x0038,
                ShortRedFirestick = 0x0039,
                BurningBarrel = 0x0046,

                EvilEye = 0x0029,
                FloatingSkull = 0x002A,

                HangingTwitchingVictim = 0x0031,
                HangingTwitchingVictim2 = 0x003F,
                HangingVictimArmsOut = 0x0032,
                HangingVictimArmsOut2 = 0x003B,
                HangingLegs = 0x0034,
                HangingLegs2 = 0x003C,
                HangingOneLeggedVictim = 0x0033,
                HangingOneLeggedVictim2 = 0x003D,
                HangingLeg = 0x0035,
                HangingLeg2 = 0x003E,
                HangingGuttedVictim = 0x0049,
                HangingGuttedDebrainedVictim = 0x004A,
                HangingTorsoLookingDown = 0x004B,
                HangingTorsoOpenBrain= 0x004C,
                HangingTorsoLookingUp = 0x004D,
                HangingTorsoDebrained = 0x004E,

                ImpaledHuman = 0x0019,
                ImpaledHumanTwitching = 0x001A,
                SkullOnPole = 0x001B,
                FiveSkullGroup = 0x001C,
                PileOfSkullsAndCandles = 0x001D,
                BloodyMess = 0x000A,
                BloodyMess2 = 0x00C,
                PoolOfBloodAndFlesh = 0x0018,
                PoolOfBlood = 0x004F,
                PoolOfBlood2 = 0x0050,
                PoolOfBrains = 0x0051,
                DeadPlayer = 0x000F,
                DeadFormerHuman = 0x0012,
                DeadFormerSergent = 0x0013,
                DeadImp = 0x0014,
                DeadDemon = 0x0015,
                DeadCacodemon = 0x0016,
                DeadLostSoul = 0x0017

            };

            private string _SpriteName;
            public string SpriteName {
                get {
                    if (_SpriteName != null) return _SpriteName;
                    switch (this.Type) {
                        case ThingType.Nothing:
                            _SpriteName = ""; break;
                        case ThingType.Nothing2:
                            _SpriteName = ""; break;
                        case ThingType.Player1Start:
                            _SpriteName = "PLAY"; break;
                        case ThingType.Player2Start:
                            _SpriteName = "PLAY"; break;
                        case ThingType.Player3Start:
                            _SpriteName = "PLAY"; break;
                        case ThingType.Player4Start:
                            _SpriteName = "PLAY"; break;
                        case ThingType.DeathmatchStart:
                            _SpriteName = ""; break;
                        case ThingType.TeleportLanding:
                            _SpriteName = ""; break;
                        case ThingType.FormerHuman:
                            _SpriteName = "POSS"; break;
                        case ThingType.WolfensteinSS:
                            _SpriteName = "SSWV"; break;
                        case ThingType.FormerHumanSergeant:
                            _SpriteName = "SPOS"; break;
                        case ThingType.HeavyWeaponDude:
                            _SpriteName = "CPOS"; break;
                        case ThingType.Imp:
                            _SpriteName = "TROO"; break;
                        case ThingType.Demon:
                            _SpriteName = "SARG"; break;
                        case ThingType.Spectre:
                            _SpriteName = "SARG"; break;
                        case ThingType.LostSoul:
                            _SpriteName = "SKUL"; break;
                        case ThingType.Cacodemon:
                            _SpriteName = "HEAD"; break;
                        case ThingType.HellKnight:
                            _SpriteName = "BOS2"; break;
                        case ThingType.BaronOfHell:
                            _SpriteName = "BOSS"; break;
                        case ThingType.Arachnotron:
                            _SpriteName = "BSPI"; break;
                        case ThingType.PainElemental:
                            _SpriteName = "PAIN"; break;
                        case ThingType.Revenant:
                            _SpriteName = "SKEL"; break;
                        case ThingType.Mancubus:
                            _SpriteName = "FATT"; break;
                        case ThingType.ArchVile:
                            _SpriteName = "VILE"; break;
                        case ThingType.SpiderMastermind:
                            _SpriteName = "SPID"; break;
                        case ThingType.CyberDemon:
                            _SpriteName = "CYBR"; break;
                        case ThingType.BossBrain:
                            _SpriteName = "BBRN"; break;
                        case ThingType.BossShooter:
                            _SpriteName = ""; break;
                        case ThingType.BossSpawnSpot:
                            _SpriteName = ""; break;
                        case ThingType.Chainsaw:
                            _SpriteName = "CSAW"; break;
                        case ThingType.Shotgun:
                            _SpriteName = "SHOT"; break;
                        case ThingType.SuperShotgun:
                            _SpriteName = "SGN2"; break;
                        case ThingType.Chaingun:
                            _SpriteName = "MGUN"; break;
                        case ThingType.RocketLauncher:
                            _SpriteName = "LAUN"; break;
                        case ThingType.PlasmaGun:
                            _SpriteName = "PLAS"; break;
                        case ThingType.Bfg9000:
                            _SpriteName = "BFUG"; break;
                        case ThingType.AmmoClip:
                            _SpriteName = "CLIP"; break;
                        case ThingType.ShotgunShells:
                            _SpriteName = "SHEL"; break;
                        case ThingType.Rocket:
                            _SpriteName = "ROCK"; break;
                        case ThingType.CellCharge:
                            _SpriteName = "CELL"; break;
                        case ThingType.AmmoCrate:
                            _SpriteName = "AMMO"; break;
                        case ThingType.ShellCrate:
                            _SpriteName = "SBOX"; break;
                        case ThingType.RocketCrate:
                            _SpriteName = "BROK"; break;
                        case ThingType.CellChargePack:
                            _SpriteName = "CELP"; break;
                        case ThingType.Backpack:
                            _SpriteName = "BPAK"; break;
                        case ThingType.Stimpack:
                            _SpriteName = "STIM"; break;
                        case ThingType.Medikit:
                            _SpriteName = "MEDI"; break;
                        case ThingType.HealthPotion:
                            _SpriteName = "BON1"; break;
                        case ThingType.SpiritArmour:
                            _SpriteName = "BON2"; break;
                        case ThingType.GreenArmour:
                            _SpriteName = "ARM1"; break;
                        case ThingType.BlueArmour:
                            _SpriteName = "ARM2"; break;
                        case ThingType.MegaSphere:
                            _SpriteName = "MEGA"; break;
                        case ThingType.SoulSphere:
                            _SpriteName = "SOUL"; break;
                        case ThingType.Invulnerability:
                            _SpriteName = "PINV"; break;
                        case ThingType.Berserk:
                            _SpriteName = "PSTR"; break;
                        case ThingType.Invisibility:
                            _SpriteName = "PINS"; break;
                        case ThingType.RadiationSuit:
                            _SpriteName = "SUIT"; break;
                        case ThingType.ComputerMap:
                            _SpriteName = "PMAP"; break;
                        case ThingType.LightAmplificationGoggles:
                            _SpriteName = "PVIS"; break;
                        case ThingType.BlueKeycard:
                            _SpriteName = "BKEY"; break;
                        case ThingType.BlueSkullkey:
                            _SpriteName = "BSKU"; break;
                        case ThingType.RedKeycard:
                            _SpriteName = "RKEY"; break;
                        case ThingType.RedSkullkey:
                            _SpriteName = "RSKU"; break;
                        case ThingType.YellowKeycard:
                            _SpriteName = "YKEY"; break;
                        case ThingType.YellowSkullkey:
                            _SpriteName = "YSKU"; break;
                        case ThingType.Barrel:
                            _SpriteName = "BAR1"; break;
                        case ThingType.CommanderKeen:
                            _SpriteName = "KEEN"; break;
                        case ThingType.TallTechnoPillar:
                            _SpriteName = "ELEC"; break;
                        case ThingType.TallGreenPillar:
                            _SpriteName = "COL1"; break;
                        case ThingType.TallRedPillar:
                            _SpriteName = "COL2"; break;
                        case ThingType.ShortGreenPillar:
                            _SpriteName = "COL3"; break;
                        case ThingType.ShortGreenPillarWithHeart:
                            _SpriteName = "COL4"; break;
                        case ThingType.ShortRedPillar:
                            _SpriteName = "COL5"; break;
                        case ThingType.ShortRedPillarWithSkull:
                            _SpriteName = "COL6"; break;
                        case ThingType.Stalagmite:
                            _SpriteName = "SMIT"; break;
                        case ThingType.BurntTree:
                            _SpriteName = "TRE1"; break;
                        case ThingType.LargeBrownTree:
                            _SpriteName = "TRE2"; break;
                        case ThingType.FloorLamp:
                            _SpriteName = "COLU"; break;
                        case ThingType.TallTechnoFloorLamp:
                            _SpriteName = "TLMP"; break;
                        case ThingType.ShortTechnoFloorLamp:
                            _SpriteName = "TLP2"; break;
                        case ThingType.Candle:
                            _SpriteName = "CAND"; break;
                        case ThingType.Candelabra:
                            _SpriteName = "CBRA"; break;
                        case ThingType.TallBlueFirestick:
                            _SpriteName = "TBLU"; break;
                        case ThingType.TallGreenFirestick:
                            _SpriteName = "TGRE"; break;
                        case ThingType.TallRedFirestick:
                            _SpriteName = "TRED"; break;
                        case ThingType.ShortBlueFirestick:
                            _SpriteName = "SMBT"; break;
                        case ThingType.ShortGreenFirestick:
                            _SpriteName = "SMGT"; break;
                        case ThingType.ShortRedFirestick:
                            _SpriteName = "SMRT"; break;
                        case ThingType.BurningBarrel:
                            _SpriteName = "FCAN"; break;
                        case ThingType.EvilEye:
                            _SpriteName = "CEYE"; break;
                        case ThingType.FloatingSkull:
                            _SpriteName = "FSKU"; break;
                        case ThingType.HangingTwitchingVictim:
                            _SpriteName = "GOR1"; break;
                        case ThingType.HangingTwitchingVictim2:
                            _SpriteName = "GOR1"; break;
                        case ThingType.HangingVictimArmsOut:
                            _SpriteName = "GOR2"; break;
                        case ThingType.HangingVictimArmsOut2:
                            _SpriteName = "GOR2"; break;
                        case ThingType.HangingLegs:
                            _SpriteName = "GOR4"; break;
                        case ThingType.HangingLegs2:
                            _SpriteName = "GOR4"; break;
                        case ThingType.HangingOneLeggedVictim:
                            _SpriteName = "GOR3"; break;
                        case ThingType.HangingOneLeggedVictim2:
                            _SpriteName = "GOR3"; break;
                        case ThingType.HangingLeg:
                            _SpriteName = "GOR5"; break;
                        case ThingType.HangingLeg2:
                            _SpriteName = "GOR5"; break;
                        case ThingType.HangingGuttedVictim:
                            _SpriteName = "HDB1"; break;
                        case ThingType.HangingGuttedDebrainedVictim:
                            _SpriteName = "HDB2"; break;
                        case ThingType.HangingTorsoLookingDown:
                            _SpriteName = "HDB3"; break;
                        case ThingType.HangingTorsoOpenBrain:
                            _SpriteName = "HDB4"; break;
                        case ThingType.HangingTorsoLookingUp:
                            _SpriteName = "HDB5"; break;
                        case ThingType.HangingTorsoDebrained:
                            _SpriteName = "HDB6"; break;
                        case ThingType.ImpaledHuman:
                            _SpriteName = "POL1"; break;
                        case ThingType.ImpaledHumanTwitching:
                            _SpriteName = "POL6"; break;
                        case ThingType.SkullOnPole:
                            _SpriteName = "POL4"; break;
                        case ThingType.FiveSkullGroup:
                            _SpriteName = "POL2"; break;
                        case ThingType.PileOfSkullsAndCandles:
                            _SpriteName = "POL3"; break;
                        case ThingType.BloodyMess:
                            _SpriteName = "PLAY"; break;
                        case ThingType.BloodyMess2:
                            _SpriteName = "PLAY"; break;
                        case ThingType.PoolOfBloodAndFlesh:
                            _SpriteName = "POL5"; break;
                        case ThingType.PoolOfBlood:
                            _SpriteName = "POB1"; break;
                        case ThingType.PoolOfBlood2:
                            _SpriteName = "POB2"; break;
                        case ThingType.PoolOfBrains:
                            _SpriteName = "BRS1"; break;
                        case ThingType.DeadPlayer:
                            _SpriteName = "PLAY"; break;
                        case ThingType.DeadFormerHuman:
                            _SpriteName = "POSS"; break;
                        case ThingType.DeadFormerSergent:
                            _SpriteName = "SPOS"; break;
                        case ThingType.DeadImp:
                            _SpriteName = "TROO"; break;
                        case ThingType.DeadDemon:
                            _SpriteName = "SARG"; break;
                        case ThingType.DeadCacodemon:
                            _SpriteName = "HEAD"; break;
                        case ThingType.DeadLostSoul:
                            _SpriteName = "SKUL"; break;
                    }
                    return _SpriteName;
                }
            }

            [Flags]
            public enum ThingOptions {
                Skills1And2 = 0x0001,
                Skill3 = 0x0002,
                Skills4And5 = 0x0004,
                Deaf = 0x0008,
                MultiplayerOnly = 0x0010
            };

            

            public Thing(int ID, Point Position, int Angle, ThingType Type, ThingOptions Options) {
                this.ID = ID;
                this.Position = Position;
                this.Angle = Angle;
                this.Type = Type;
                this.Options = Options;
            }

        }
    }
}